# -*- coding: utf-8 -*-
from ....pipeline import engine as pe
from ....interfaces import utility as util
from ....interfaces import cmtk as cmtk
from ....algorithms import misc as misc
from ....algorithms.misc import remove_identical_paths
from .group_connectivity import pullnodeIDs


def add_global_to_filename(in_file):
    from nipype.utils.filemanip import split_filename
    path, name, ext = split_filename(in_file)
    return name + '_global' + ext


def add_nodal_to_filename(in_file):
    from nipype.utils.filemanip import split_filename
    path, name, ext = split_filename(in_file)
    return name + '_nodal' + ext


def create_networkx_pipeline(name="networkx", extra_column_heading="subject"):
    """Creates a workflow to calculate various graph measures (via NetworkX) on
    an input network. The output measures are then converted to comma-separated value
    text files, and an extra column / field is also added. Typically, the user would
    connect the subject name to this field.

    Example
    -------

    >>> from nipype.workflows.dmri.connectivity.nx import create_networkx_pipeline
    >>> nx = create_networkx_pipeline("networkx", "subject_id")
    >>> nx.inputs.inputnode.extra_field = 'subj1'
    >>> nx.inputs.inputnode.network_file = 'subj1.pck'
    >>> nx.run()                 # doctest: +SKIP

    Inputs::

        inputnode.extra_field
        inputnode.network_file

    Outputs::

        outputnode.network_files
        outputnode.csv_files
        outputnode.matlab_files

    """
    inputnode = pe.Node(
        interface=util.IdentityInterface(
            fields=["extra_field", "network_file"]),
        name="inputnode")

    pipeline = pe.Workflow(name=name)

    ntwkMetrics = pe.Node(
        interface=cmtk.NetworkXMetrics(), name="NetworkXMetrics")
    Matlab2CSV_node = pe.Node(
        interface=misc.Matlab2CSV(), name="Matlab2CSV_node")
    MergeCSVFiles_node = pe.Node(
        interface=misc.MergeCSVFiles(), name="MergeCSVFiles_node")
    MergeCSVFiles_node.inputs.extra_column_heading = extra_column_heading

    Matlab2CSV_global = Matlab2CSV_node.clone(name="Matlab2CSV_global")
    MergeCSVFiles_global = MergeCSVFiles_node.clone(
        name="MergeCSVFiles_global")
    MergeCSVFiles_global.inputs.extra_column_heading = extra_column_heading

    mergeNetworks = pe.Node(interface=util.Merge(2), name="mergeNetworks")
    mergeCSVs = mergeNetworks.clone("mergeCSVs")

    pipeline.connect([(inputnode, ntwkMetrics, [("network_file", "in_file")])])
    pipeline.connect([(ntwkMetrics, Matlab2CSV_node, [("node_measures_matlab",
                                                       "in_file")])])
    pipeline.connect([(ntwkMetrics, Matlab2CSV_global,
                       [("global_measures_matlab", "in_file")])])

    pipeline.connect([(Matlab2CSV_node, MergeCSVFiles_node, [("csv_files",
                                                              "in_files")])])
    pipeline.connect([(inputnode, MergeCSVFiles_node,
                       [(("extra_field", add_nodal_to_filename),
                         "out_file")])])
    pipeline.connect([(inputnode, MergeCSVFiles_node, [("extra_field",
                                                        "extra_field")])])
    pipeline.connect([(inputnode, MergeCSVFiles_node,
                       [(("network_file", pullnodeIDs), "row_headings")])])

    pipeline.connect([(Matlab2CSV_global, MergeCSVFiles_global,
                       [("csv_files", "in_files")])])
    pipeline.connect([(Matlab2CSV_global, MergeCSVFiles_global,
                       [(("csv_files", remove_identical_paths),
                         "column_headings")])])
    # MergeCSVFiles_global.inputs.row_heading_title = 'metric'
    # MergeCSVFiles_global.inputs.column_headings = ['average']

    pipeline.connect([(inputnode, MergeCSVFiles_global,
                       [(("extra_field", add_global_to_filename),
                         "out_file")])])
    pipeline.connect([(inputnode, MergeCSVFiles_global, [("extra_field",
                                                          "extra_field")])])

    pipeline.connect([(inputnode, mergeNetworks, [("network_file", "in1")])])
    pipeline.connect([(ntwkMetrics, mergeNetworks, [("gpickled_network_files",
                                                     "in2")])])

    outputnode = pe.Node(
        interface=util.IdentityInterface(fields=[
            "network_files", "csv_files", "matlab_files", "node_csv",
            "global_csv"
        ]),
        name="outputnode")

    pipeline.connect([(MergeCSVFiles_node, outputnode, [("csv_file",
                                                         "node_csv")])])
    pipeline.connect([(MergeCSVFiles_global, outputnode, [("csv_file",
                                                           "global_csv")])])

    pipeline.connect([(MergeCSVFiles_node, mergeCSVs, [("csv_file", "in1")])])
    pipeline.connect([(MergeCSVFiles_global, mergeCSVs, [("csv_file",
                                                          "in2")])])
    pipeline.connect([(mergeNetworks, outputnode, [("out", "network_files")])])
    pipeline.connect([(mergeCSVs, outputnode, [("out", "csv_files")])])
    pipeline.connect([(ntwkMetrics, outputnode, [("matlab_matrix_files",
                                                  "matlab_files")])])
    return pipeline


def create_cmats_to_csv_pipeline(name="cmats_to_csv",
                                 extra_column_heading="subject"):
    """Creates a workflow to convert the outputs from CreateMatrix into a single
    comma-separated value text file. An extra column / field is also added to the
    text file. Typically, the user would connect the subject name to this field.

    Example
    -------

    >>> from nipype.workflows.dmri.connectivity.nx import create_cmats_to_csv_pipeline
    >>> csv = create_cmats_to_csv_pipeline("cmats_to_csv", "subject_id")
    >>> csv.inputs.inputnode.extra_field = 'subj1'
    >>> csv.inputs.inputnode.matlab_matrix_files = ['subj1_cmatrix.mat', 'subj1_mean_fiber_length.mat', 'subj1_median_fiber_length.mat', 'subj1_fiber_length_std.mat']
    >>> csv.run()                 # doctest: +SKIP

    Inputs::

        inputnode.extra_field
        inputnode.matlab_matrix_files

    Outputs::

        outputnode.csv_file

    """
    inputnode = pe.Node(
        interface=util.IdentityInterface(
            fields=["extra_field", "matlab_matrix_files"]),
        name="inputnode")

    pipeline = pe.Workflow(name=name)

    Matlab2CSV = pe.MapNode(
        interface=misc.Matlab2CSV(), name="Matlab2CSV", iterfield=["in_file"])
    MergeCSVFiles = pe.Node(
        interface=misc.MergeCSVFiles(), name="MergeCSVFiles")
    MergeCSVFiles.inputs.extra_column_heading = extra_column_heading

    pipeline.connect([(inputnode, Matlab2CSV, [("matlab_matrix_files",
                                                "in_file")])])
    pipeline.connect([(Matlab2CSV, MergeCSVFiles, [("csv_files",
                                                    "in_files")])])
    pipeline.connect([(inputnode, MergeCSVFiles, [("extra_field",
                                                   "extra_field")])])

    outputnode = pe.Node(
        interface=util.IdentityInterface(fields=["csv_file"]),
        name="outputnode")

    pipeline.connect([(MergeCSVFiles, outputnode, [("csv_file", "csv_file")])])
    return pipeline



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
    # changing to temporary directories
    >>> tmp = getfixture('tmpdir')
    >>> old = tmp.chdir()
"""
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)
from builtins import range

from future import standard_library
standard_library.install_aliases()

import os
import re
import numpy as np
import nibabel as nb

from ..base import (traits, TraitedSpec, DynamicTraitedSpec, File, Undefined,
                    isdefined, OutputMultiPath, InputMultiPath, BaseInterface,
                    BaseInterfaceInputSpec, Str)
from ..io import IOBase, add_traits
from ...utils.filemanip import ensure_list, copyfile, split_filename


class IdentityInterface(IOBase):
    """Basic interface class generates identity mappings

    Examples
    --------

    >>> from nipype.interfaces.utility import IdentityInterface
    >>> ii = IdentityInterface(fields=['a', 'b'], mandatory_inputs=False)
    >>> ii.inputs.a
    <undefined>

    >>> ii.inputs.a = 'foo'
    >>> out = ii._outputs()
    >>> out.a
    <undefined>

    >>> out = ii.run()
    >>> out.outputs.a
    'foo'

    >>> ii2 = IdentityInterface(fields=['a', 'b'], mandatory_inputs=True)
    >>> ii2.inputs.a = 'foo'
    >>> out = ii2.run() # doctest: +SKIP
    ValueError: IdentityInterface requires a value for input 'b' because it was listed in 'fields' Interface IdentityInterface failed to run.
    """
    input_spec = DynamicTraitedSpec
    output_spec = DynamicTraitedSpec

    def __init__(self, fields=None, mandatory_inputs=True, **inputs):
        super(IdentityInterface, self).__init__(**inputs)
        if fields is None or not fields:
            raise ValueError(
                'Identity Interface fields must be a non-empty list')
        # Each input must be in the fields.
        for in_field in inputs:
            if in_field not in fields:
                raise ValueError(
                    'Identity Interface input is not in the fields: %s' %
                    in_field)
        self._fields = fields
        self._mandatory_inputs = mandatory_inputs
        add_traits(self.inputs, fields)
        # Adding any traits wipes out all input values set in superclass initialization,
        # even it the trait is not in the add_traits argument. The work-around is to reset
        # the values after adding the traits.
        self.inputs.trait_set(**inputs)

    def _add_output_traits(self, base):
        return add_traits(base, self._fields)

    def _list_outputs(self):
        # manual mandatory inputs check
        if self._fields and self._mandatory_inputs:
            for key in self._fields:
                value = getattr(self.inputs, key)
                if not isdefined(value):
                    msg = "%s requires a value for input '%s' because it was listed in 'fields'. \
                    You can turn off mandatory inputs checking by passing mandatory_inputs = False to the constructor."                                                                                                                                                                                                                                               % \
                        (self.__class__.__name__, key)
                    raise ValueError(msg)

        outputs = self._outputs().get()
        for key in self._fields:
            val = getattr(self.inputs, key)
            if isdefined(val):
                outputs[key] = val
        return outputs


class MergeInputSpec(DynamicTraitedSpec, BaseInterfaceInputSpec):
    axis = traits.Enum(
        'vstack',
        'hstack',
        usedefault=True,
        desc=
        'direction in which to merge, hstack requires same number of elements in each input'
    )
    no_flatten = traits.Bool(
        False,
        usedefault=True,
        desc='append to outlist instead of extending in vstack mode')
    ravel_inputs = traits.Bool(
        False, usedefault=True, desc='ravel inputs when no_flatten is False')


class MergeOutputSpec(TraitedSpec):
    out = traits.List(desc='Merged output')


def _ravel(in_val):
    if not isinstance(in_val, list):
        return in_val
    flat_list = []
    for val in in_val:
        raveled_val = _ravel(val)
        if isinstance(raveled_val, list):
            flat_list.extend(raveled_val)
        else:
            flat_list.append(raveled_val)
    return flat_list


class Merge(IOBase):
    """Basic interface class to merge inputs into a single list

    ``Merge(1)`` will merge a list of lists

    Examples
    --------

    >>> from nipype.interfaces.utility import Merge
    >>> mi = Merge(3)
    >>> mi.inputs.in1 = 1
    >>> mi.inputs.in2 = [2, 5]
    >>> mi.inputs.in3 = 3
    >>> out = mi.run()
    >>> out.outputs.out
    [1, 2, 5, 3]

    >>> merge = Merge(1)
    >>> merge.inputs.in1 = [1, [2, 5], 3]
    >>> out = merge.run()
    >>> out.outputs.out
    [1, [2, 5], 3]

    >>> merge = Merge(1)
    >>> merge.inputs.in1 = [1, [2, 5], 3]
    >>> merge.inputs.ravel_inputs = True
    >>> out = merge.run()
    >>> out.outputs.out
    [1, 2, 5, 3]

    >>> merge = Merge(1)
    >>> merge.inputs.in1 = [1, [2, 5], 3]
    >>> merge.inputs.no_flatten = True
    >>> out = merge.run()
    >>> out.outputs.out
    [[1, [2, 5], 3]]
    """
    input_spec = MergeInputSpec
    output_spec = MergeOutputSpec

    def __init__(self, numinputs=0, **inputs):
        super(Merge, self).__init__(**inputs)
        self._numinputs = numinputs
        if numinputs >= 1:
            input_names = ['in%d' % (i + 1) for i in range(numinputs)]
        else:
            input_names = []
        add_traits(self.inputs, input_names)

    def _list_outputs(self):
        outputs = self._outputs().get()
        out = []

        if self._numinputs < 1:
            return outputs
        else:
            getval = lambda idx: getattr(self.inputs, 'in%d' % (idx + 1))
            values = [
                getval(idx) for idx in range(self._numinputs)
                if isdefined(getval(idx))
            ]

        if self.inputs.axis == 'vstack':
            for value in values:
                if isinstance(value, list) and not self.inputs.no_flatten:
                    out.extend(
                        _ravel(value) if self.inputs.ravel_inputs else value)
                else:
                    out.append(value)
        else:
            lists = [ensure_list(val) for val in values]
            out = [[val[i] for val in lists] for i in range(len(lists[0]))]
        outputs['out'] = out
        return outputs


class RenameInputSpec(DynamicTraitedSpec):

    in_file = File(exists=True, mandatory=True, desc="file to rename")
    keep_ext = traits.Bool(
        desc=("Keep in_file extension, replace "
              "non-extension component of name"))
    format_string = Str(
        mandatory=True, desc="Python formatting string for output template")
    parse_string = Str(desc="Python regexp parse string to define "
                       "replacement inputs")
    use_fullpath = traits.Bool(
        False, usedefault=True, desc="Use full path as input to regex parser")


class RenameOutputSpec(TraitedSpec):

    out_file = traits.File(
        exists=True, desc="softlink to original file with new name")


class Rename(IOBase):
    """Change the name of a file based on a mapped format string.

    To use additional inputs that will be defined at run-time, the class
    constructor must be called with the format template, and the fields
    identified will become inputs to the interface.

    Additionally, you may set the parse_string input, which will be run
    over the input filename with a regular expressions search, and will
    fill in additional input fields from matched groups. Fields set with
    inputs have precedence over fields filled in with the regexp match.

    Examples
    --------

    >>> from nipype.interfaces.utility import Rename
    >>> rename1 = Rename()
    >>> rename1.inputs.in_file = os.path.join(datadir, "zstat1.nii.gz") # datadir is a directory with exemplary files, defined in conftest.py
    >>> rename1.inputs.format_string = "Faces-Scenes.nii.gz"
    >>> res = rename1.run()          # doctest: +SKIP
    >>> res.outputs.out_file         # doctest: +SKIP
    'Faces-Scenes.nii.gz"            # doctest: +SKIP

    >>> rename2 = Rename(format_string="%(subject_id)s_func_run%(run)02d")
    >>> rename2.inputs.in_file = os.path.join(datadir, "functional.nii")
    >>> rename2.inputs.keep_ext = True
    >>> rename2.inputs.subject_id = "subj_201"
    >>> rename2.inputs.run = 2
    >>> res = rename2.run()          # doctest: +SKIP
    >>> res.outputs.out_file         # doctest: +SKIP
    'subj_201_func_run02.nii'        # doctest: +SKIP

    >>> rename3 = Rename(format_string="%(subject_id)s_%(seq)s_run%(run)02d.nii")
    >>> rename3.inputs.in_file = os.path.join(datadir, "func_epi_1_1.nii")
    >>> rename3.inputs.parse_string = "func_(?P<seq>\w*)_.*"
    >>> rename3.inputs.subject_id = "subj_201"
    >>> rename3.inputs.run = 2
    >>> res = rename3.run()          # doctest: +SKIP
    >>> res.outputs.out_file         # doctest: +SKIP
    'subj_201_epi_run02.nii'         # doctest: +SKIP

    """
    input_spec = RenameInputSpec
    output_spec = RenameOutputSpec

    def __init__(self, format_string=None, **inputs):
        super(Rename, self).__init__(**inputs)
        if format_string is not None:
            self.inputs.format_string = format_string
            self.fmt_fields = re.findall(r"%\((.+?)\)", format_string)
            add_traits(self.inputs, self.fmt_fields)
        else:
            self.fmt_fields = []

    def _rename(self):
        fmt_dict = dict()
        if isdefined(self.inputs.parse_string):
            if isdefined(
                    self.inputs.use_fullpath) and self.inputs.use_fullpath:
                m = re.search(self.inputs.parse_string, self.inputs.in_file)
            else:
                m = re.search(self.inputs.parse_string,
                              os.path.split(self.inputs.in_file)[1])
            if m:
                fmt_dict.update(m.groupdict())
        for field in self.fmt_fields:
            val = getattr(self.inputs, field)
            if isdefined(val):
                fmt_dict[field] = getattr(self.inputs, field)
        if self.inputs.keep_ext:
            fmt_string = "".join([
                self.inputs.format_string,
                split_filename(self.inputs.in_file)[2]
            ])
        else:
            fmt_string = self.inputs.format_string
        return fmt_string % fmt_dict

    def _run_interface(self, runtime):
        runtime.returncode = 0
        _ = copyfile(self.inputs.in_file,
                     os.path.join(os.getcwd(), self._rename()))
        return runtime

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs["out_file"] = os.path.join(os.getcwd(), self._rename())
        return outputs


class SplitInputSpec(BaseInterfaceInputSpec):
    inlist = traits.List(
        traits.Any, mandatory=True, desc='list of values to split')
    splits = traits.List(
        traits.Int,
        mandatory=True,
        desc='Number of outputs in each split - should add to number of inputs'
    )
    squeeze = traits.Bool(
        False,
        usedefault=True,
        desc='unfold one-element splits removing the list')


class Split(IOBase):
    """Basic interface class to split lists into multiple outputs

    Examples
    --------

    >>> from nipype.interfaces.utility import Split
    >>> sp = Split()
    >>> _ = sp.inputs.trait_set(inlist=[1, 2, 3], splits=[2, 1])
    >>> out = sp.run()
    >>> out.outputs.out1
    [1, 2]

    """

    input_spec = SplitInputSpec
    output_spec = DynamicTraitedSpec

    def _add_output_traits(self, base):
        undefined_traits = {}
        for i in range(len(self.inputs.splits)):
            key = 'out%d' % (i + 1)
            base.add_trait(key, traits.Any)
            undefined_traits[key] = Undefined
        base.trait_set(trait_change_notify=False, **undefined_traits)
        return base

    def _list_outputs(self):
        outputs = self._outputs().get()
        if isdefined(self.inputs.splits):
            if sum(self.inputs.splits) != len(self.inputs.inlist):
                raise RuntimeError('sum of splits != num of list elements')
            splits = [0]
            splits.extend(self.inputs.splits)
            splits = np.cumsum(splits)
            for i in range(len(splits) - 1):
                val = np.array(
                    self.inputs.inlist)[splits[i]:splits[i + 1]].tolist()
                if self.inputs.squeeze and len(val) == 1:
                    val = val[0]
                outputs['out%d' % (i + 1)] = val
        return outputs


class SelectInputSpec(BaseInterfaceInputSpec):
    inlist = InputMultiPath(
        traits.Any, mandatory=True, desc='list of values to choose from')
    index = InputMultiPath(
        traits.Int, mandatory=True, desc='0-based indices of values to choose')


class SelectOutputSpec(TraitedSpec):
    out = OutputMultiPath(traits.Any, desc='list of selected values')


class Select(IOBase):
    """Basic interface class to select specific elements from a list

    Examples
    --------

    >>> from nipype.interfaces.utility import Select
    >>> sl = Select()
    >>> _ = sl.inputs.trait_set(inlist=[1, 2, 3, 4, 5], index=[3])
    >>> out = sl.run()
    >>> out.outputs.out
    4

    >>> _ = sl.inputs.trait_set(inlist=[1, 2, 3, 4, 5], index=[3, 4])
    >>> out = sl.run()
    >>> out.outputs.out
    [4, 5]

    """

    input_spec = SelectInputSpec
    output_spec = SelectOutputSpec

    def _list_outputs(self):
        outputs = self._outputs().get()
        out = np.array(self.inputs.inlist)[np.array(
            self.inputs.index)].tolist()
        outputs['out'] = out
        return outputs


class AssertEqualInputSpec(BaseInterfaceInputSpec):
    volume1 = File(exists=True, mandatory=True)
    volume2 = File(exists=True, mandatory=True)


class AssertEqual(BaseInterface):
    input_spec = AssertEqualInputSpec

    def _run_interface(self, runtime):

        data1 = nb.load(self.inputs.volume1).get_data()
        data2 = nb.load(self.inputs.volume2).get_data()

        if not np.all(data1 == data2):
            raise RuntimeError('Input images are not exactly equal')
        return runtime



# -*- coding: utf-8 -*-
from ....interfaces import utility as util  # utility
from ....pipeline import engine as pe  # pypeline engine
from ....interfaces import fsl as fsl
from ....interfaces import mrtrix as mrtrix


def create_mrtrix_dti_pipeline(name="dtiproc",
                               tractography_type='probabilistic'):
    """Creates a pipeline that does the same diffusion processing as in the
    :doc:`../../users/examples/dmri_mrtrix_dti` example script. Given a diffusion-weighted image,
    b-values, and b-vectors, the workflow will return the tractography
    computed from spherical deconvolution and probabilistic streamline tractography

    Example
    -------

    >>> dti = create_mrtrix_dti_pipeline("mrtrix_dti")
    >>> dti.inputs.inputnode.dwi = 'data.nii'
    >>> dti.inputs.inputnode.bvals = 'bvals'
    >>> dti.inputs.inputnode.bvecs = 'bvecs'
    >>> dti.run()                  # doctest: +SKIP

    Inputs::

        inputnode.dwi
        inputnode.bvecs
        inputnode.bvals

    Outputs::

        outputnode.fa
        outputnode.tdi
        outputnode.tracts_tck
        outputnode.tracts_trk
        outputnode.csdeconv

    """

    inputnode = pe.Node(
        interface=util.IdentityInterface(fields=["dwi", "bvecs", "bvals"]),
        name="inputnode")

    bet = pe.Node(interface=fsl.BET(), name="bet")
    bet.inputs.mask = True

    fsl2mrtrix = pe.Node(interface=mrtrix.FSL2MRTrix(), name='fsl2mrtrix')
    fsl2mrtrix.inputs.invert_y = True

    dwi2tensor = pe.Node(interface=mrtrix.DWI2Tensor(), name='dwi2tensor')

    tensor2vector = pe.Node(
        interface=mrtrix.Tensor2Vector(), name='tensor2vector')
    tensor2adc = pe.Node(
        interface=mrtrix.Tensor2ApparentDiffusion(), name='tensor2adc')
    tensor2fa = pe.Node(
        interface=mrtrix.Tensor2FractionalAnisotropy(), name='tensor2fa')

    erode_mask_firstpass = pe.Node(
        interface=mrtrix.Erode(), name='erode_mask_firstpass')
    erode_mask_secondpass = pe.Node(
        interface=mrtrix.Erode(), name='erode_mask_secondpass')

    threshold_b0 = pe.Node(interface=mrtrix.Threshold(), name='threshold_b0')

    threshold_FA = pe.Node(interface=mrtrix.Threshold(), name='threshold_FA')
    threshold_FA.inputs.absolute_threshold_value = 0.7

    threshold_wmmask = pe.Node(
        interface=mrtrix.Threshold(), name='threshold_wmmask')
    threshold_wmmask.inputs.absolute_threshold_value = 0.4

    MRmultiply = pe.Node(interface=mrtrix.MRMultiply(), name='MRmultiply')
    MRmult_merge = pe.Node(interface=util.Merge(2), name='MRmultiply_merge')

    median3d = pe.Node(interface=mrtrix.MedianFilter3D(), name='median3D')

    MRconvert = pe.Node(interface=mrtrix.MRConvert(), name='MRconvert')
    MRconvert.inputs.extract_at_axis = 3
    MRconvert.inputs.extract_at_coordinate = [0]

    csdeconv = pe.Node(
        interface=mrtrix.ConstrainedSphericalDeconvolution(), name='csdeconv')

    gen_WM_mask = pe.Node(
        interface=mrtrix.GenerateWhiteMatterMask(), name='gen_WM_mask')

    estimateresponse = pe.Node(
        interface=mrtrix.EstimateResponseForSH(), name='estimateresponse')

    if tractography_type == 'probabilistic':
        CSDstreamtrack = pe.Node(
            interface=mrtrix.
            ProbabilisticSphericallyDeconvolutedStreamlineTrack(),
            name='CSDstreamtrack')
    else:
        CSDstreamtrack = pe.Node(
            interface=mrtrix.SphericallyDeconvolutedStreamlineTrack(),
            name='CSDstreamtrack')
    CSDstreamtrack.inputs.desired_number_of_tracks = 15000

    tracks2prob = pe.Node(interface=mrtrix.Tracks2Prob(), name='tracks2prob')
    tracks2prob.inputs.colour = True
    tck2trk = pe.Node(interface=mrtrix.MRTrix2TrackVis(), name='tck2trk')

    workflow = pe.Workflow(name=name)
    workflow.base_output_dir = name

    workflow.connect([(inputnode, fsl2mrtrix, [("bvecs", "bvec_file"),
                                               ("bvals", "bval_file")])])
    workflow.connect([(inputnode, dwi2tensor, [("dwi", "in_file")])])
    workflow.connect([(fsl2mrtrix, dwi2tensor, [("encoding_file",
                                                 "encoding_file")])])

    workflow.connect([
        (dwi2tensor, tensor2vector, [['tensor', 'in_file']]),
        (dwi2tensor, tensor2adc, [['tensor', 'in_file']]),
        (dwi2tensor, tensor2fa, [['tensor', 'in_file']]),
    ])

    workflow.connect([(inputnode, MRconvert, [("dwi", "in_file")])])
    workflow.connect([(MRconvert, threshold_b0, [("converted", "in_file")])])
    workflow.connect([(threshold_b0, median3d, [("out_file", "in_file")])])
    workflow.connect([(median3d, erode_mask_firstpass, [("out_file",
                                                         "in_file")])])
    workflow.connect([(erode_mask_firstpass, erode_mask_secondpass,
                       [("out_file", "in_file")])])

    workflow.connect([(tensor2fa, MRmult_merge, [("FA", "in1")])])
    workflow.connect([(erode_mask_secondpass, MRmult_merge, [("out_file",
                                                              "in2")])])
    workflow.connect([(MRmult_merge, MRmultiply, [("out", "in_files")])])
    workflow.connect([(MRmultiply, threshold_FA, [("out_file", "in_file")])])
    workflow.connect([(threshold_FA, estimateresponse, [("out_file",
                                                         "mask_image")])])

    workflow.connect([(inputnode, bet, [("dwi", "in_file")])])
    workflow.connect([(inputnode, gen_WM_mask, [("dwi", "in_file")])])
    workflow.connect([(bet, gen_WM_mask, [("mask_file", "binary_mask")])])
    workflow.connect([(fsl2mrtrix, gen_WM_mask, [("encoding_file",
                                                  "encoding_file")])])

    workflow.connect([(inputnode, estimateresponse, [("dwi", "in_file")])])
    workflow.connect([(fsl2mrtrix, estimateresponse, [("encoding_file",
                                                       "encoding_file")])])

    workflow.connect([(inputnode, csdeconv, [("dwi", "in_file")])])
    workflow.connect([(gen_WM_mask, csdeconv, [("WMprobabilitymap",
                                                "mask_image")])])
    workflow.connect([(estimateresponse, csdeconv, [("response",
                                                     "response_file")])])
    workflow.connect([(fsl2mrtrix, csdeconv, [("encoding_file",
                                               "encoding_file")])])

    workflow.connect([(gen_WM_mask, threshold_wmmask, [("WMprobabilitymap",
                                                        "in_file")])])
    workflow.connect([(threshold_wmmask, CSDstreamtrack, [("out_file",
                                                           "seed_file")])])
    workflow.connect([(csdeconv, CSDstreamtrack, [("spherical_harmonics_image",
                                                   "in_file")])])

    if tractography_type == 'probabilistic':
        workflow.connect([(CSDstreamtrack, tracks2prob, [("tracked",
                                                          "in_file")])])
        workflow.connect([(inputnode, tracks2prob, [("dwi",
                                                     "template_file")])])

    workflow.connect([(CSDstreamtrack, tck2trk, [("tracked", "in_file")])])
    workflow.connect([(inputnode, tck2trk, [("dwi", "image_file")])])

    output_fields = ["fa", "tracts_trk", "csdeconv", "tracts_tck"]
    if tractography_type == 'probabilistic':
        output_fields.append("tdi")
    outputnode = pe.Node(
        interface=util.IdentityInterface(fields=output_fields),
        name="outputnode")

    workflow.connect([(CSDstreamtrack, outputnode,
                       [("tracked", "tracts_tck")]), (csdeconv, outputnode, [
                           ("spherical_harmonics_image", "csdeconv")
                       ]), (tensor2fa, outputnode, [("FA", "fa")]),
                      (tck2trk, outputnode, [("out_file", "tracts_trk")])])
    if tractography_type == 'probabilistic':
        workflow.connect([(tracks2prob, outputnode, [("tract_image", "tdi")])])

    return workflow



# -*- coding: utf-8 -*-
"""ANTS Apply Transforms interface
"""
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)
from builtins import range, str
import os

from .base import ANTSCommand, ANTSCommandInputSpec
from ..base import TraitedSpec, File, traits, isdefined, InputMultiPath
from ...utils.filemanip import split_filename


class WarpTimeSeriesImageMultiTransformInputSpec(ANTSCommandInputSpec):
    dimension = traits.Enum(
        4,
        3,
        argstr='%d',
        usedefault=True,
        desc='image dimension (3 or 4)',
        position=1)
    input_image = File(
        argstr='%s',
        mandatory=True,
        copyfile=True,
        desc=('image to apply transformation to (generally a '
              'coregistered functional)'))
    out_postfix = traits.Str(
        '_wtsimt',
        argstr='%s',
        usedefault=True,
        desc=('Postfix that is prepended to all output '
              'files (default = _wtsimt)'))
    reference_image = File(
        argstr='-R %s',
        xor=['tightest_box'],
        desc='reference image space that you wish to warp INTO')
    tightest_box = traits.Bool(
        argstr='--tightest-bounding-box',
        desc=('computes tightest bounding box (overrided by '
              'reference_image if given)'),
        xor=['reference_image'])
    reslice_by_header = traits.Bool(
        argstr='--reslice-by-header',
        desc=('Uses orientation matrix and origin encoded in '
              'reference image file header. Not typically used '
              'with additional transforms'))
    use_nearest = traits.Bool(
        argstr='--use-NN', desc='Use nearest neighbor interpolation')
    use_bspline = traits.Bool(
        argstr='--use-Bspline', desc='Use 3rd order B-Spline interpolation')
    transformation_series = InputMultiPath(
        File(exists=True),
        argstr='%s',
        desc='transformation file(s) to be applied',
        mandatory=True,
        copyfile=False)
    invert_affine = traits.List(
        traits.Int,
        desc=(
            'List of Affine transformations to invert.'
            'E.g.: [1,4,5] inverts the 1st, 4th, and 5th Affines '
            'found in transformation_series. Note that indexing '
            'starts with 1 and does not include warp fields. Affine '
            'transformations are distinguished '
            'from warp fields by the word "affine" included in their filenames.'
        ))


class WarpTimeSeriesImageMultiTransformOutputSpec(TraitedSpec):
    output_image = File(exists=True, desc='Warped image')


class WarpTimeSeriesImageMultiTransform(ANTSCommand):
    """Warps a time-series from one space to another

    Examples
    --------

    >>> from nipype.interfaces.ants import WarpTimeSeriesImageMultiTransform
    >>> wtsimt = WarpTimeSeriesImageMultiTransform()
    >>> wtsimt.inputs.input_image = 'resting.nii'
    >>> wtsimt.inputs.reference_image = 'ants_deformed.nii.gz'
    >>> wtsimt.inputs.transformation_series = ['ants_Warp.nii.gz','ants_Affine.txt']
    >>> wtsimt.cmdline
    'WarpTimeSeriesImageMultiTransform 4 resting.nii resting_wtsimt.nii -R ants_deformed.nii.gz ants_Warp.nii.gz \
ants_Affine.txt'

    >>> wtsimt = WarpTimeSeriesImageMultiTransform()
    >>> wtsimt.inputs.input_image = 'resting.nii'
    >>> wtsimt.inputs.reference_image = 'ants_deformed.nii.gz'
    >>> wtsimt.inputs.transformation_series = ['ants_Warp.nii.gz','ants_Affine.txt']
    >>> wtsimt.inputs.invert_affine = [1] # # this will invert the 1st Affine file: ants_Affine.txt
    >>> wtsimt.cmdline
    'WarpTimeSeriesImageMultiTransform 4 resting.nii resting_wtsimt.nii -R ants_deformed.nii.gz ants_Warp.nii.gz \
-i ants_Affine.txt'
    """

    _cmd = 'WarpTimeSeriesImageMultiTransform'
    input_spec = WarpTimeSeriesImageMultiTransformInputSpec
    output_spec = WarpTimeSeriesImageMultiTransformOutputSpec

    def _format_arg(self, opt, spec, val):
        if opt == 'out_postfix':
            _, name, ext = split_filename(
                os.path.abspath(self.inputs.input_image))
            return name + val + ext
        if opt == 'transformation_series':
            series = []
            affine_counter = 0
            affine_invert = []
            for transformation in val:
                if 'Affine' in transformation and \
                        isdefined(self.inputs.invert_affine):
                    affine_counter += 1
                    if affine_counter in self.inputs.invert_affine:
                        series += ['-i']
                        affine_invert.append(affine_counter)
                series += [transformation]

            if isdefined(self.inputs.invert_affine):
                diff_inv = set(self.inputs.invert_affine) - set(affine_invert)
                if diff_inv:
                    raise Exceptions(
                        "Review invert_affine, not all indexes from invert_affine were used, "
                        "check the description for the full definition")

            return ' '.join(series)
        return super(WarpTimeSeriesImageMultiTransform, self)._format_arg(
            opt, spec, val)

    def _list_outputs(self):
        outputs = self._outputs().get()
        _, name, ext = split_filename(os.path.abspath(self.inputs.input_image))
        outputs['output_image'] = os.path.join(os.getcwd(), ''.join(
            (name, self.inputs.out_postfix, ext)))
        return outputs

    def _run_interface(self, runtime, correct_return_codes=[0]):
        runtime = super(WarpTimeSeriesImageMultiTransform,
                        self)._run_interface(
                            runtime, correct_return_codes=[0, 1])
        if "100 % complete" not in runtime.stdout:
            self.raise_exception(runtime)
        return runtime


class WarpImageMultiTransformInputSpec(ANTSCommandInputSpec):
    dimension = traits.Enum(
        3,
        2,
        argstr='%d',
        usedefault=True,
        desc='image dimension (2 or 3)',
        position=1)
    input_image = File(
        argstr='%s',
        mandatory=True,
        desc=('image to apply transformation to (generally a '
              'coregistered functional)'),
        position=2)
    output_image = File(
        genfile=True,
        hash_files=False,
        argstr='%s',
        desc='name of the output warped image',
        position=3,
        xor=['out_postfix'])
    out_postfix = File(
        "_wimt",
        usedefault=True,
        hash_files=False,
        desc=('Postfix that is prepended to all output '
              'files (default = _wimt)'),
        xor=['output_image'])
    reference_image = File(
        argstr='-R %s',
        xor=['tightest_box'],
        desc='reference image space that you wish to warp INTO')
    tightest_box = traits.Bool(
        argstr='--tightest-bounding-box',
        desc=('computes tightest bounding box (overrided by '
              'reference_image if given)'),
        xor=['reference_image'])
    reslice_by_header = traits.Bool(
        argstr='--reslice-by-header',
        desc=('Uses orientation matrix and origin encoded in '
              'reference image file header. Not typically used '
              'with additional transforms'))
    use_nearest = traits.Bool(
        argstr='--use-NN', desc='Use nearest neighbor interpolation')
    use_bspline = traits.Bool(
        argstr='--use-BSpline', desc='Use 3rd order B-Spline interpolation')
    transformation_series = InputMultiPath(
        File(exists=True),
        argstr='%s',
        desc='transformation file(s) to be applied',
        mandatory=True,
        position=-1)
    invert_affine = traits.List(
        traits.Int,
        desc=(
            'List of Affine transformations to invert.'
            'E.g.: [1,4,5] inverts the 1st, 4th, and 5th Affines '
            'found in transformation_series. Note that indexing '
            'starts with 1 and does not include warp fields. Affine '
            'transformations are distinguished '
            'from warp fields by the word "affine" included in their filenames.'
        ))


class WarpImageMultiTransformOutputSpec(TraitedSpec):
    output_image = File(exists=True, desc='Warped image')


class WarpImageMultiTransform(ANTSCommand):
    """Warps an image from one space to another

    Examples
    --------

    >>> from nipype.interfaces.ants import WarpImageMultiTransform
    >>> wimt = WarpImageMultiTransform()
    >>> wimt.inputs.input_image = 'structural.nii'
    >>> wimt.inputs.reference_image = 'ants_deformed.nii.gz'
    >>> wimt.inputs.transformation_series = ['ants_Warp.nii.gz','ants_Affine.txt']
    >>> wimt.cmdline
    'WarpImageMultiTransform 3 structural.nii structural_wimt.nii -R ants_deformed.nii.gz ants_Warp.nii.gz \
ants_Affine.txt'

    >>> wimt = WarpImageMultiTransform()
    >>> wimt.inputs.input_image = 'diffusion_weighted.nii'
    >>> wimt.inputs.reference_image = 'functional.nii'
    >>> wimt.inputs.transformation_series = ['func2anat_coreg_Affine.txt','func2anat_InverseWarp.nii.gz', \
    'dwi2anat_Warp.nii.gz','dwi2anat_coreg_Affine.txt']
    >>> wimt.inputs.invert_affine = [1]  # this will invert the 1st Affine file: 'func2anat_coreg_Affine.txt'
    >>> wimt.cmdline
    'WarpImageMultiTransform 3 diffusion_weighted.nii diffusion_weighted_wimt.nii -R functional.nii \
-i func2anat_coreg_Affine.txt func2anat_InverseWarp.nii.gz dwi2anat_Warp.nii.gz dwi2anat_coreg_Affine.txt'

    """

    _cmd = 'WarpImageMultiTransform'
    input_spec = WarpImageMultiTransformInputSpec
    output_spec = WarpImageMultiTransformOutputSpec

    def _gen_filename(self, name):
        if name == 'output_image':
            _, name, ext = split_filename(
                os.path.abspath(self.inputs.input_image))
            return ''.join((name, self.inputs.out_postfix, ext))
        return None

    def _format_arg(self, opt, spec, val):
        if opt == 'transformation_series':
            series = []
            affine_counter = 0
            affine_invert = []
            for transformation in val:
                if "affine" in transformation.lower() and \
                        isdefined(self.inputs.invert_affine):
                    affine_counter += 1
                    if affine_counter in self.inputs.invert_affine:
                        series += ['-i']
                        affine_invert.append(affine_counter)
                series += [transformation]

            if isdefined(self.inputs.invert_affine):
                diff_inv = set(self.inputs.invert_affine) - set(affine_invert)
                if diff_inv:
                    raise Exceptions(
                        "Review invert_affine, not all indexes from invert_affine were used, "
                        "check the description for the full definition")

            return ' '.join(series)

        return super(WarpImageMultiTransform, self)._format_arg(opt, spec, val)

    def _list_outputs(self):
        outputs = self._outputs().get()
        if isdefined(self.inputs.output_image):
            outputs['output_image'] = os.path.abspath(self.inputs.output_image)
        else:
            outputs['output_image'] = os.path.abspath(
                self._gen_filename('output_image'))
        return outputs


class ApplyTransformsInputSpec(ANTSCommandInputSpec):
    dimension = traits.Enum(
        2,
        3,
        4,
        argstr='--dimensionality %d',
        desc=('This option forces the image to be treated '
              'as a specified-dimensional image. If not '
              'specified, antsWarp tries to infer the '
              'dimensionality from the input image.'))
    input_image_type = traits.Enum(
        0,
        1,
        2,
        3,
        argstr='--input-image-type %d',
        desc=('Option specifying the input image '
              'type of scalar (default), vector, '
              'tensor, or time series.'))
    input_image = File(
        argstr='--input %s',
        mandatory=True,
        desc=('image to apply transformation to (generally a '
              'coregistered functional)'),
        exists=True)
    output_image = traits.Str(
        argstr='--output %s',
        desc='output file name',
        genfile=True,
        hash_files=False)
    out_postfix = traits.Str(
        "_trans",
        usedefault=True,
        desc=('Postfix that is appended to all output '
              'files (default = _trans)'))
    reference_image = File(
        argstr='--reference-image %s',
        mandatory=True,
        desc='reference image space that you wish to warp INTO',
        exists=True)
    interpolation = traits.Enum(
        'Linear',
        'NearestNeighbor',
        'CosineWindowedSinc',
        'WelchWindowedSinc',
        'HammingWindowedSinc',
        'LanczosWindowedSinc',
        'MultiLabel',
        'Gaussian',
        'BSpline',
        argstr='%s',
        usedefault=True)
    interpolation_parameters = traits.Either(
        traits.Tuple(traits.Int()),  # BSpline (order)
        traits.Tuple(
            traits.Float(),  # Gaussian/MultiLabel (sigma, alpha)
            traits.Float()))
    transforms = traits.Either(
        InputMultiPath(File(exists=True)),
        'identity',
        argstr='%s',
        mandatory=True,
        desc='transform files: will be applied in reverse order. For '
        'example, the last specified transform will be applied first.')
    invert_transform_flags = InputMultiPath(traits.Bool())
    default_value = traits.Float(
        0.0, argstr='--default-value %g', usedefault=True)
    print_out_composite_warp_file = traits.Bool(
        False,
        requires=["output_image"],
        desc='output a composite warp file instead of a transformed image')
    float = traits.Bool(
        argstr='--float %d',
        default_value=False,
        usedefault=True,
        desc='Use float instead of double for computations.')


class ApplyTransformsOutputSpec(TraitedSpec):
    output_image = File(exists=True, desc='Warped image')


class ApplyTransforms(ANTSCommand):
    """ApplyTransforms, applied to an input image, transforms it according to a
    reference image and a transform (or a set of transforms).

    Examples
    --------

    >>> from nipype.interfaces.ants import ApplyTransforms
    >>> at = ApplyTransforms()
    >>> at.inputs.input_image = 'moving1.nii'
    >>> at.inputs.reference_image = 'fixed1.nii'
    >>> at.inputs.transforms = 'identity'
    >>> at.cmdline
    'antsApplyTransforms --default-value 0 --float 0 --input moving1.nii \
--interpolation Linear --output moving1_trans.nii \
--reference-image fixed1.nii -t identity'

    >>> at = ApplyTransforms()
    >>> at.inputs.dimension = 3
    >>> at.inputs.input_image = 'moving1.nii'
    >>> at.inputs.reference_image = 'fixed1.nii'
    >>> at.inputs.output_image = 'deformed_moving1.nii'
    >>> at.inputs.interpolation = 'Linear'
    >>> at.inputs.default_value = 0
    >>> at.inputs.transforms = ['ants_Warp.nii.gz', 'trans.mat']
    >>> at.inputs.invert_transform_flags = [False, False]
    >>> at.cmdline
    'antsApplyTransforms --default-value 0 --dimensionality 3 --float 0 --input moving1.nii \
--interpolation Linear --output deformed_moving1.nii --reference-image fixed1.nii \
--transform [ ants_Warp.nii.gz, 0 ] --transform [ trans.mat, 0 ]'

    >>> at1 = ApplyTransforms()
    >>> at1.inputs.dimension = 3
    >>> at1.inputs.input_image = 'moving1.nii'
    >>> at1.inputs.reference_image = 'fixed1.nii'
    >>> at1.inputs.output_image = 'deformed_moving1.nii'
    >>> at1.inputs.interpolation = 'BSpline'
    >>> at1.inputs.interpolation_parameters = (5,)
    >>> at1.inputs.default_value = 0
    >>> at1.inputs.transforms = ['ants_Warp.nii.gz', 'trans.mat']
    >>> at1.inputs.invert_transform_flags = [False, False]
    >>> at1.cmdline
    'antsApplyTransforms --default-value 0 --dimensionality 3 --float 0 --input moving1.nii \
--interpolation BSpline[ 5 ] --output deformed_moving1.nii --reference-image fixed1.nii \
--transform [ ants_Warp.nii.gz, 0 ] --transform [ trans.mat, 0 ]'
    """
    _cmd = 'antsApplyTransforms'
    input_spec = ApplyTransformsInputSpec
    output_spec = ApplyTransformsOutputSpec

    def _gen_filename(self, name):
        if name == 'output_image':
            output = self.inputs.output_image
            if not isdefined(output):
                _, name, ext = split_filename(self.inputs.input_image)
                output = name + self.inputs.out_postfix + ext
            return output
        return None

    def _get_transform_filenames(self):
        retval = []
        for ii in range(len(self.inputs.transforms)):
            if isdefined(self.inputs.invert_transform_flags):
                if len(self.inputs.transforms) == len(
                        self.inputs.invert_transform_flags):
                    invert_code = 1 if self.inputs.invert_transform_flags[
                        ii] else 0
                    retval.append("--transform [ %s, %d ]" %
                                  (self.inputs.transforms[ii], invert_code))
                else:
                    raise Exception((
                        "ERROR: The useInverse list must have the same number "
                        "of entries as the transformsFileName list."))
            else:
                retval.append("--transform %s" % self.inputs.transforms[ii])
        return " ".join(retval)

    def _get_output_warped_filename(self):
        if isdefined(self.inputs.print_out_composite_warp_file):
            return "--output [ %s, %d ]" % (
                self._gen_filename("output_image"),
                int(self.inputs.print_out_composite_warp_file))
        else:
            return "--output %s" % (self._gen_filename("output_image"))

    def _format_arg(self, opt, spec, val):
        if opt == "output_image":
            return self._get_output_warped_filename()
        elif opt == "transforms":
            if val == 'identity':
                return '-t identity'
            return self._get_transform_filenames()
        elif opt == 'interpolation':
            if self.inputs.interpolation in ['BSpline', 'MultiLabel', 'Gaussian'] and \
                    isdefined(self.inputs.interpolation_parameters):
                return '--interpolation %s[ %s ]' % (
                    self.inputs.interpolation, ', '.join([
                        str(param)
                        for param in self.inputs.interpolation_parameters
                    ]))
            else:
                return '--interpolation %s' % self.inputs.interpolation
        return super(ApplyTransforms, self)._format_arg(opt, spec, val)

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs['output_image'] = os.path.abspath(
            self._gen_filename('output_image'))
        return outputs


class ApplyTransformsToPointsInputSpec(ANTSCommandInputSpec):
    dimension = traits.Enum(
        2,
        3,
        4,
        argstr='--dimensionality %d',
        desc=('This option forces the image to be treated '
              'as a specified-dimensional image. If not '
              'specified, antsWarp tries to infer the '
              'dimensionality from the input image.'))
    input_file = File(
        argstr='--input %s',
        mandatory=True,
        desc=
        ("Currently, the only input supported is a csv file with "
         "columns including x,y (2D), x,y,z (3D) or x,y,z,t,label (4D) column headers."
         "The points should be defined in physical space."
         "If in doubt how to convert coordinates from your files to the space"
         "required by antsApplyTransformsToPoints try creating/drawing a simple"
         "label volume with only one voxel set to 1 and all others set to 0."
         "Write down the voxel coordinates. Then use ImageMaths LabelStats to find"
         "out what coordinates for this voxel antsApplyTransformsToPoints is"
         "expecting."),
        exists=True)
    output_file = traits.Str(
        argstr='--output %s',
        desc='Name of the output CSV file',
        name_source=['input_file'],
        hash_files=False,
        name_template='%s_transformed.csv')
    transforms = traits.List(
        File(exists=True),
        argstr='%s',
        mandatory=True,
        desc='transforms that will be applied to the points')
    invert_transform_flags = traits.List(
        traits.Bool(),
        desc='list indicating if a transform should be reversed')


class ApplyTransformsToPointsOutputSpec(TraitedSpec):
    output_file = File(
        exists=True, desc='csv file with transformed coordinates')


class ApplyTransformsToPoints(ANTSCommand):
    """ApplyTransformsToPoints, applied to an CSV file, transforms coordinates
    using provided transform (or a set of transforms).

    Examples
    --------

    >>> from nipype.interfaces.ants import ApplyTransforms
    >>> at = ApplyTransformsToPoints()
    >>> at.inputs.dimension = 3
    >>> at.inputs.input_file = 'moving.csv'
    >>> at.inputs.transforms = ['trans.mat', 'ants_Warp.nii.gz']
    >>> at.inputs.invert_transform_flags = [False, False]
    >>> at.cmdline
    'antsApplyTransformsToPoints --dimensionality 3 --input moving.csv --output moving_transformed.csv \
--transform [ trans.mat, 0 ] --transform [ ants_Warp.nii.gz, 0 ]'


    """
    _cmd = 'antsApplyTransformsToPoints'
    input_spec = ApplyTransformsToPointsInputSpec
    output_spec = ApplyTransformsToPointsOutputSpec

    def _get_transform_filenames(self):
        retval = []
        for ii in range(len(self.inputs.transforms)):
            if isdefined(self.inputs.invert_transform_flags):
                if len(self.inputs.transforms) == len(
                        self.inputs.invert_transform_flags):
                    invert_code = 1 if self.inputs.invert_transform_flags[
                        ii] else 0
                    retval.append("--transform [ %s, %d ]" %
                                  (self.inputs.transforms[ii], invert_code))
                else:
                    raise Exception((
                        "ERROR: The useInverse list must have the same number "
                        "of entries as the transformsFileName list."))
            else:
                retval.append("--transform %s" % self.inputs.transforms[ii])
        return " ".join(retval)

    def _format_arg(self, opt, spec, val):
        if opt == "transforms":
            return self._get_transform_filenames()
        return super(ApplyTransformsToPoints, self)._format_arg(opt, spec, val)



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:

from .groupwise import (create_groupwise_average, create_nonlinear_gw_step,
                        create_linear_gw_step)



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""Tests for join expansion
"""
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)
from builtins import open

from ... import engine as pe
from ....interfaces import base as nib
from ....interfaces.utility import IdentityInterface, Function, Merge
from ....interfaces.base import traits, File


class PickFirstSpec(nib.TraitedSpec):
    in_files = traits.List(
        File(exists=True), argstr="%s", position=2, mandatory=True)


class PickFirstOutSpec(nib.TraitedSpec):
    output1 = File(exists=True)


class PickFirst(nib.BaseInterface):
    input_spec = PickFirstSpec
    output_spec = PickFirstOutSpec

    def _run_interface(self, runtime):
        runtime.returncode = 0
        return runtime

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs['output1'] = self.inputs.in_files[0]
        return outputs


class IncrementInputSpec(nib.TraitedSpec):
    input1 = nib.traits.Int(mandatory=True, desc='input')
    inc = nib.traits.Int(usedefault=True, default_value=1, desc='increment')


class IncrementOutputSpec(nib.TraitedSpec):
    output1 = nib.traits.Int(desc='ouput')


class IncrementInterface(nib.BaseInterface):
    input_spec = IncrementInputSpec
    output_spec = IncrementOutputSpec

    def _run_interface(self, runtime):
        runtime.returncode = 0
        return runtime

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs['output1'] = self.inputs.input1 + self.inputs.inc
        return outputs


_sums = []

_sum_operands = []


class SumInputSpec(nib.TraitedSpec):
    input1 = nib.traits.List(nib.traits.Int, mandatory=True, desc='input')


class SumOutputSpec(nib.TraitedSpec):
    output1 = nib.traits.Int(desc='ouput')
    operands = nib.traits.List(nib.traits.Int, desc='operands')


class SumInterface(nib.BaseInterface):
    input_spec = SumInputSpec
    output_spec = SumOutputSpec

    def _run_interface(self, runtime):
        runtime.returncode = 0
        return runtime

    def _list_outputs(self):
        global _sum
        global _sum_operands
        outputs = self._outputs().get()
        outputs['operands'] = self.inputs.input1
        _sum_operands.append(outputs['operands'])
        outputs['output1'] = sum(self.inputs.input1)
        _sums.append(outputs['output1'])
        return outputs


_set_len = None
"""The Set interface execution result."""


class SetInputSpec(nib.TraitedSpec):
    input1 = nib.traits.Set(nib.traits.Int, mandatory=True, desc='input')


class SetOutputSpec(nib.TraitedSpec):
    output1 = nib.traits.Int(desc='ouput')


class SetInterface(nib.BaseInterface):
    input_spec = SetInputSpec
    output_spec = SetOutputSpec

    def _run_interface(self, runtime):
        runtime.returncode = 0
        return runtime

    def _list_outputs(self):
        global _set_len
        outputs = self._outputs().get()
        _set_len = outputs['output1'] = len(self.inputs.input1)
        return outputs


_products = []
"""The Products interface execution results."""


class ProductInputSpec(nib.TraitedSpec):
    input1 = nib.traits.Int(mandatory=True, desc='input1')
    input2 = nib.traits.Int(mandatory=True, desc='input2')


class ProductOutputSpec(nib.TraitedSpec):
    output1 = nib.traits.Int(mandatory=True, desc='output')


class ProductInterface(nib.BaseInterface):
    input_spec = ProductInputSpec
    output_spec = ProductOutputSpec

    def _run_interface(self, runtime):
        runtime.returncode = 0
        return runtime

    def _list_outputs(self):
        global _products
        outputs = self._outputs().get()
        outputs['output1'] = self.inputs.input1 * self.inputs.input2
        _products.append(outputs['output1'])
        return outputs


def test_join_expansion(tmpdir):
    tmpdir.chdir()

    # Make the workflow.
    wf = pe.Workflow(name='test')
    # the iterated input node
    inputspec = pe.Node(IdentityInterface(fields=['n']), name='inputspec')
    inputspec.iterables = [('n', [1, 2])]
    # a pre-join node in the iterated path
    pre_join1 = pe.Node(IncrementInterface(), name='pre_join1')
    wf.connect(inputspec, 'n', pre_join1, 'input1')
    # another pre-join node in the iterated path
    pre_join2 = pe.Node(IncrementInterface(), name='pre_join2')
    wf.connect(pre_join1, 'output1', pre_join2, 'input1')
    # the join node
    join = pe.JoinNode(
        SumInterface(),
        joinsource='inputspec',
        joinfield='input1',
        name='join')
    wf.connect(pre_join2, 'output1', join, 'input1')
    # an uniterated post-join node
    post_join1 = pe.Node(IncrementInterface(), name='post_join1')
    wf.connect(join, 'output1', post_join1, 'input1')
    # a post-join node in the iterated path
    post_join2 = pe.Node(ProductInterface(), name='post_join2')
    wf.connect(join, 'output1', post_join2, 'input1')
    wf.connect(pre_join1, 'output1', post_join2, 'input2')

    result = wf.run()

    # the two expanded pre-join predecessor nodes feed into one join node
    joins = [node for node in result.nodes() if node.name == 'join']
    assert len(joins) == 1, "The number of join result nodes is incorrect."
    # the expanded graph contains 2 * 2 = 4 iteration pre-join nodes, 1 join
    # node, 1 non-iterated post-join node and 2 * 1 iteration post-join nodes.
    # Nipype factors away the IdentityInterface.
    assert len(
        result.nodes()) == 8, "The number of expanded nodes is incorrect."
    # the join Sum result is (1 + 1 + 1) + (2 + 1 + 1)
    assert len(_sums) == 1, "The number of join outputs is incorrect"
    assert _sums[
        0] == 7, "The join Sum output value is incorrect: %s." % _sums[0]
    # the join input preserves the iterables input order
    assert _sum_operands[0] == [3, 4], \
        "The join Sum input is incorrect: %s." % _sum_operands[0]
    # there are two iterations of the post-join node in the iterable path
    assert len(_products) == 2,\
        "The number of iterated post-join outputs is incorrect"


def test_node_joinsource(tmpdir):
    """Test setting the joinsource to a Node."""
    tmpdir.chdir()

    # the iterated input node
    inputspec = pe.Node(IdentityInterface(fields=['n']), name='inputspec')
    inputspec.iterables = [('n', [1, 2])]
    # the join node
    join = pe.JoinNode(
        SetInterface(), joinsource=inputspec, joinfield='input1', name='join')

    # the joinsource is the inputspec name
    assert join.joinsource == inputspec.name, \
        "The joinsource is not set to the node name."


def test_set_join_node(tmpdir):
    """Test collecting join inputs to a set."""
    tmpdir.chdir()

    # Make the workflow.
    wf = pe.Workflow(name='test')
    # the iterated input node
    inputspec = pe.Node(IdentityInterface(fields=['n']), name='inputspec')
    inputspec.iterables = [('n', [1, 2, 1, 3, 2])]
    # a pre-join node in the iterated path
    pre_join1 = pe.Node(IncrementInterface(), name='pre_join1')
    wf.connect(inputspec, 'n', pre_join1, 'input1')
    # the set join node
    join = pe.JoinNode(
        SetInterface(),
        joinsource='inputspec',
        joinfield='input1',
        name='join')
    wf.connect(pre_join1, 'output1', join, 'input1')

    wf.run()

    # the join length is the number of unique inputs
    assert _set_len == 3, \
        "The join Set output value is incorrect: %s." % _set_len


def test_unique_join_node(tmpdir):
    """Test join with the ``unique`` flag set to True."""
    global _sum_operands
    _sum_operands = []
    tmpdir.chdir()

    # Make the workflow.
    wf = pe.Workflow(name='test')
    # the iterated input node
    inputspec = pe.Node(IdentityInterface(fields=['n']), name='inputspec')
    inputspec.iterables = [('n', [3, 1, 2, 1, 3])]
    # a pre-join node in the iterated path
    pre_join1 = pe.Node(IncrementInterface(), name='pre_join1')
    wf.connect(inputspec, 'n', pre_join1, 'input1')
    # the set join node
    join = pe.JoinNode(
        SumInterface(),
        joinsource='inputspec',
        joinfield='input1',
        unique=True,
        name='join')
    wf.connect(pre_join1, 'output1', join, 'input1')

    wf.run()

    assert _sum_operands[0] == [4, 2, 3], \
        "The unique join output value is incorrect: %s." % _sum_operands[0]


def test_multiple_join_nodes(tmpdir):
    """Test two join nodes, one downstream of the other."""
    global _products
    _products = []
    tmpdir.chdir()

    # Make the workflow.
    wf = pe.Workflow(name='test')
    # the iterated input node
    inputspec = pe.Node(IdentityInterface(fields=['n']), name='inputspec')
    inputspec.iterables = [('n', [1, 2, 3])]
    # a pre-join node in the iterated path
    pre_join1 = pe.Node(IncrementInterface(), name='pre_join1')
    wf.connect(inputspec, 'n', pre_join1, 'input1')
    # the first join node
    join1 = pe.JoinNode(
        IdentityInterface(fields=['vector']),
        joinsource='inputspec',
        joinfield='vector',
        name='join1')
    wf.connect(pre_join1, 'output1', join1, 'vector')
    # an uniterated post-join node
    post_join1 = pe.Node(SumInterface(), name='post_join1')
    wf.connect(join1, 'vector', post_join1, 'input1')
    # the downstream join node connected to both an upstream join
    # path output and a separate input in the iterated path
    join2 = pe.JoinNode(
        IdentityInterface(fields=['vector', 'scalar']),
        joinsource='inputspec',
        joinfield='vector',
        name='join2')
    wf.connect(pre_join1, 'output1', join2, 'vector')
    wf.connect(post_join1, 'output1', join2, 'scalar')
    # a second post-join node
    post_join2 = pe.Node(SumInterface(), name='post_join2')
    wf.connect(join2, 'vector', post_join2, 'input1')
    # a third post-join node
    post_join3 = pe.Node(ProductInterface(), name='post_join3')
    wf.connect(post_join2, 'output1', post_join3, 'input1')
    wf.connect(join2, 'scalar', post_join3, 'input2')

    result = wf.run()

    # The expanded graph contains one pre_join1 replicate per inputspec
    # replicate and one of each remaining node = 3 + 5 = 8 nodes.
    # The replicated inputspec nodes are factored out of the expansion.
    assert len(result.nodes()) == 8, \
        "The number of expanded nodes is incorrect."
    # The outputs are:
    # pre_join1: [2, 3, 4]
    # post_join1: 9
    # join2: [2, 3, 4] and 9
    # post_join2: 9
    # post_join3: 9 * 9 = 81
    assert _products == [81], "The post-join product is incorrect"


def test_identity_join_node(tmpdir):
    """Test an IdentityInterface join."""
    global _sum_operands
    _sum_operands = []
    tmpdir.chdir()

    # Make the workflow.
    wf = pe.Workflow(name='test')
    # the iterated input node
    inputspec = pe.Node(IdentityInterface(fields=['n']), name='inputspec')
    inputspec.iterables = [('n', [1, 2, 3])]
    # a pre-join node in the iterated path
    pre_join1 = pe.Node(IncrementInterface(), name='pre_join1')
    wf.connect(inputspec, 'n', pre_join1, 'input1')
    # the IdentityInterface join node
    join = pe.JoinNode(
        IdentityInterface(fields=['vector']),
        joinsource='inputspec',
        joinfield='vector',
        name='join')
    wf.connect(pre_join1, 'output1', join, 'vector')
    # an uniterated post-join node
    post_join1 = pe.Node(SumInterface(), name='post_join1')
    wf.connect(join, 'vector', post_join1, 'input1')

    result = wf.run()

    # the expanded graph contains 1 * 3 iteration pre-join nodes, 1 join
    # node and 1 post-join node. Nipype factors away the iterable input
    # IdentityInterface but keeps the join IdentityInterface.
    assert len(result.nodes()) == 5, \
        "The number of expanded nodes is incorrect."
    assert _sum_operands[0] == [2, 3, 4], \
        "The join Sum input is incorrect: %s." % _sum_operands[0]


def test_multifield_join_node(tmpdir):
    """Test join on several fields."""
    global _products
    _products = []
    tmpdir.chdir()

    # Make the workflow.
    wf = pe.Workflow(name='test')
    # the iterated input node
    inputspec = pe.Node(IdentityInterface(fields=['m', 'n']), name='inputspec')
    inputspec.iterables = [('m', [1, 2]), ('n', [3, 4])]
    # two pre-join nodes in a parallel iterated path
    inc1 = pe.Node(IncrementInterface(), name='inc1')
    wf.connect(inputspec, 'm', inc1, 'input1')
    inc2 = pe.Node(IncrementInterface(), name='inc2')
    wf.connect(inputspec, 'n', inc2, 'input1')
    # the join node
    join = pe.JoinNode(
        IdentityInterface(fields=['vector1', 'vector2']),
        joinsource='inputspec',
        name='join')
    wf.connect(inc1, 'output1', join, 'vector1')
    wf.connect(inc2, 'output1', join, 'vector2')
    # a post-join node
    prod = pe.MapNode(
        ProductInterface(), name='prod', iterfield=['input1', 'input2'])
    wf.connect(join, 'vector1', prod, 'input1')
    wf.connect(join, 'vector2', prod, 'input2')

    result = wf.run()

    # the iterables are expanded as the cartesian product of the iterables values.
    # thus, the expanded graph contains 2 * (2 * 2) iteration pre-join nodes, 1 join
    # node and 1 post-join node.
    assert len(result.nodes()) == 10, \
        "The number of expanded nodes is incorrect."
    # the product inputs are [2, 4], [2, 5], [3, 4], [3, 5]
    assert set(_products) == set([8, 10, 12, 15]), \
        "The post-join products is incorrect: %s." % _products


def test_synchronize_join_node(tmpdir):
    """Test join on an input node which has the ``synchronize`` flag set to True."""
    global _products
    _products = []
    tmpdir.chdir()

    # Make the workflow.
    wf = pe.Workflow(name='test')
    # the iterated input node
    inputspec = pe.Node(IdentityInterface(fields=['m', 'n']), name='inputspec')
    inputspec.iterables = [('m', [1, 2]), ('n', [3, 4])]
    inputspec.synchronize = True
    # two pre-join nodes in a parallel iterated path
    inc1 = pe.Node(IncrementInterface(), name='inc1')
    wf.connect(inputspec, 'm', inc1, 'input1')
    inc2 = pe.Node(IncrementInterface(), name='inc2')
    wf.connect(inputspec, 'n', inc2, 'input1')
    # the join node
    join = pe.JoinNode(
        IdentityInterface(fields=['vector1', 'vector2']),
        joinsource='inputspec',
        name='join')
    wf.connect(inc1, 'output1', join, 'vector1')
    wf.connect(inc2, 'output1', join, 'vector2')
    # a post-join node
    prod = pe.MapNode(
        ProductInterface(), name='prod', iterfield=['input1', 'input2'])
    wf.connect(join, 'vector1', prod, 'input1')
    wf.connect(join, 'vector2', prod, 'input2')

    result = wf.run()

    # there are 3 iterables expansions.
    # thus, the expanded graph contains 2 * 2 iteration pre-join nodes, 1 join
    # node and 1 post-join node.
    assert len(result.nodes()) == 6, \
        "The number of expanded nodes is incorrect."
    # the product inputs are [2, 3] and [4, 5]
    assert _products == [8, 15], \
        "The post-join products is incorrect: %s." % _products


def test_itersource_join_source_node(tmpdir):
    """Test join on an input node which has an ``itersource``."""
    tmpdir.chdir()

    # Make the workflow.
    wf = pe.Workflow(name='test')
    # the iterated input node
    inputspec = pe.Node(IdentityInterface(fields=['n']), name='inputspec')
    inputspec.iterables = [('n', [1, 2])]
    # an intermediate node in the first iteration path
    pre_join1 = pe.Node(IncrementInterface(), name='pre_join1')
    wf.connect(inputspec, 'n', pre_join1, 'input1')
    # an iterable pre-join node with an itersource
    pre_join2 = pe.Node(ProductInterface(), name='pre_join2')
    pre_join2.itersource = ('inputspec', 'n')
    pre_join2.iterables = ('input1', {1: [3, 4], 2: [5, 6]})
    wf.connect(pre_join1, 'output1', pre_join2, 'input2')
    # an intermediate node in the second iteration path
    pre_join3 = pe.Node(IncrementInterface(), name='pre_join3')
    wf.connect(pre_join2, 'output1', pre_join3, 'input1')
    # the join node
    join = pe.JoinNode(
        IdentityInterface(fields=['vector']),
        joinsource='pre_join2',
        joinfield='vector',
        name='join')
    wf.connect(pre_join3, 'output1', join, 'vector')
    # a join successor node
    post_join1 = pe.Node(SumInterface(), name='post_join1')
    wf.connect(join, 'vector', post_join1, 'input1')

    result = wf.run()

    # the expanded graph contains
    # 1 pre_join1 replicate for each inputspec iteration,
    # 2 pre_join2 replicates for each inputspec iteration,
    # 1 pre_join3 for each pre_join2 iteration,
    # 1 join replicate for each inputspec iteration and
    # 1 post_join1 replicate for each join replicate =
    # 2 + (2 * 2) + 4 + 2 + 2 = 14 expansion graph nodes.
    # Nipype factors away the iterable input
    # IdentityInterface but keeps the join IdentityInterface.
    assert len(result.nodes()) == 14, \
        "The number of expanded nodes is incorrect."
    # The first join inputs are:
    # 1 + (3 * 2) and 1 + (4 * 2)
    # The second join inputs are:
    # 1 + (5 * 3) and 1 + (6 * 3)
    # the post-join nodes execution order is indeterminate;
    # therefore, compare the lists item-wise.
    assert [16, 19] in _sum_operands, \
        "The join Sum input is incorrect: %s." % _sum_operands
    assert [7, 9] in _sum_operands, \
        "The join Sum input is incorrect: %s." % _sum_operands


def test_itersource_two_join_nodes(tmpdir):
    """Test join with a midstream ``itersource`` and an upstream
    iterable."""
    tmpdir.chdir()

    # Make the workflow.
    wf = pe.Workflow(name='test')
    # the iterated input node
    inputspec = pe.Node(IdentityInterface(fields=['n']), name='inputspec')
    inputspec.iterables = [('n', [1, 2])]
    # an intermediate node in the first iteration path
    pre_join1 = pe.Node(IncrementInterface(), name='pre_join1')
    wf.connect(inputspec, 'n', pre_join1, 'input1')
    # an iterable pre-join node with an itersource
    pre_join2 = pe.Node(ProductInterface(), name='pre_join2')
    pre_join2.itersource = ('inputspec', 'n')
    pre_join2.iterables = ('input1', {1: [3, 4], 2: [5, 6]})
    wf.connect(pre_join1, 'output1', pre_join2, 'input2')
    # an intermediate node in the second iteration path
    pre_join3 = pe.Node(IncrementInterface(), name='pre_join3')
    wf.connect(pre_join2, 'output1', pre_join3, 'input1')
    # the first join node
    join1 = pe.JoinNode(
        IdentityInterface(fields=['vector']),
        joinsource='pre_join2',
        joinfield='vector',
        name='join1')
    wf.connect(pre_join3, 'output1', join1, 'vector')
    # a join successor node
    post_join1 = pe.Node(SumInterface(), name='post_join1')
    wf.connect(join1, 'vector', post_join1, 'input1')
    # a summary join node
    join2 = pe.JoinNode(
        IdentityInterface(fields=['vector']),
        joinsource='inputspec',
        joinfield='vector',
        name='join2')
    wf.connect(post_join1, 'output1', join2, 'vector')

    result = wf.run()

    # the expanded graph contains the 14 test_itersource_join_source_node
    # nodes plus the summary join node.
    assert len(result.nodes()) == 15, \
        "The number of expanded nodes is incorrect."


def test_set_join_node_file_input(tmpdir):
    """Test collecting join inputs to a set."""
    tmpdir.chdir()
    open('test.nii', 'w+').close()
    open('test2.nii', 'w+').close()

    # Make the workflow.
    wf = pe.Workflow(name='test')
    # the iterated input node
    inputspec = pe.Node(IdentityInterface(fields=['n']), name='inputspec')
    inputspec.iterables = [('n', [
        tmpdir.join('test.nii').strpath,
        tmpdir.join('test2.nii').strpath
    ])]
    # a pre-join node in the iterated path
    pre_join1 = pe.Node(IdentityInterface(fields=['n']), name='pre_join1')
    wf.connect(inputspec, 'n', pre_join1, 'n')
    # the set join node
    join = pe.JoinNode(
        PickFirst(), joinsource='inputspec', joinfield='in_files', name='join')
    wf.connect(pre_join1, 'n', join, 'in_files')

    wf.run()


def test_nested_workflow_join(tmpdir):
    """Test collecting join inputs within a nested workflow"""
    tmpdir.chdir()

    # Make the nested workflow
    def nested_wf(i, name='smallwf'):
        # iterables with list of nums
        inputspec = pe.Node(IdentityInterface(fields=['n']), name='inputspec')
        inputspec.iterables = [('n', i)]
        # increment each iterable before joining
        pre_join = pe.Node(IncrementInterface(), name='pre_join')
        # rejoin nums into list
        join = pe.JoinNode(
            IdentityInterface(fields=['n']),
            joinsource='inputspec',
            joinfield='n',
            name='join')
        # define and connect nested workflow
        wf = pe.Workflow(name='wf_%d' % i[0])
        wf.connect(inputspec, 'n', pre_join, 'input1')
        wf.connect(pre_join, 'output1', join, 'n')
        return wf

    # master wf
    meta_wf = pe.Workflow(name='meta', base_dir='.')
    # add each mini-workflow to master
    for i in [[1, 3], [2, 4]]:
        mini_wf = nested_wf(i)
        meta_wf.add_nodes([mini_wf])

    result = meta_wf.run()

    # there should be six nodes in total
    assert len(result.nodes()) == 6, \
        "The number of expanded nodes is incorrect."


def test_name_prefix_join(tmpdir):
    tmpdir.chdir()

    def sq(x):
        return x ** 2

    wf = pe.Workflow('wf', base_dir=tmpdir.strpath)
    square = pe.Node(Function(function=sq), name='square')
    square.iterables = [('x', [1, 2])]
    square_join = pe.JoinNode(Merge(1, ravel_inputs=True),
                              name='square_join',
                              joinsource='square',
                              joinfield=['in1'])
    wf.connect(square, 'out', square_join, "in1")
    wf.run()



# -*- coding: utf-8 -*-
""" Base interfaces for dipy """
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)

import os.path as op
import numpy as np
from ... import logging
from ..base import (traits, File, isdefined, LibraryBaseInterface,
                    BaseInterfaceInputSpec)

HAVE_DIPY = True
try:
    import dipy
except ImportError:
    HAVE_DIPY = False


def no_dipy():
    """ Check if dipy is available """
    global HAVE_DIPY
    return not HAVE_DIPY


def dipy_version():
    """ Check dipy version """
    if no_dipy():
        return None

    return dipy.__version__


class DipyBaseInterface(LibraryBaseInterface):
    """
    A base interface for py:mod:`dipy` computations
    """
    _pkg = 'dipy'


class DipyBaseInterfaceInputSpec(BaseInterfaceInputSpec):
    in_file = File(exists=True, mandatory=True, desc=('input diffusion data'))
    in_bval = File(exists=True, mandatory=True, desc=('input b-values table'))
    in_bvec = File(exists=True, mandatory=True, desc=('input b-vectors table'))
    b0_thres = traits.Int(700, usedefault=True, desc=('b0 threshold'))
    out_prefix = traits.Str(desc=('output prefix for file names'))


class DipyDiffusionInterface(DipyBaseInterface):
    """
    A base interface for py:mod:`dipy` computations
    """
    input_spec = DipyBaseInterfaceInputSpec

    def _get_gradient_table(self):
        bval = np.loadtxt(self.inputs.in_bval)
        bvec = np.loadtxt(self.inputs.in_bvec).T
        from dipy.core.gradients import gradient_table
        gtab = gradient_table(bval, bvec)

        gtab.b0_threshold = self.inputs.b0_thres
        return gtab

    def _gen_filename(self, name, ext=None):
        fname, fext = op.splitext(op.basename(self.inputs.in_file))
        if fext == '.gz':
            fname, fext2 = op.splitext(fname)
            fext = fext2 + fext

        if not isdefined(self.inputs.out_prefix):
            out_prefix = op.abspath(fname)
        else:
            out_prefix = self.inputs.out_prefix

        if ext is None:
            ext = fext

        return out_prefix + '_' + name + ext



__author__ = 'oliver'

from ..preprocess import create_featreg_preproc, pickrun


def test_pickrun():
    files = ['1', '2', '3', '4']
    assert pickrun(files, 0) == '1'
    assert pickrun(files, 'first') == '1'
    assert pickrun(files, -1) == '4'
    assert pickrun(files, 'last') == '4'
    assert pickrun(files, 'middle') == '3'


def test_create_featreg_preproc():
    """smoke test"""
    wf = create_featreg_preproc(whichrun=0)

    # test type
    import nipype
    assert type(wf) == nipype.pipeline.engine.Workflow

    # test methods
    assert wf.get_node('extractref')
    assert wf._get_dot()



# -*- coding: utf-8 -*-
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)
from ....pipeline import engine as pe
from ....interfaces import freesurfer as fs
from ....interfaces import utility as niu
from .autorecon1 import create_AutoRecon1
from .autorecon2 import create_AutoRecon2
from .autorecon3 import create_AutoRecon3
from ....interfaces.freesurfer import AddXFormToHeader, Info
from ....interfaces.io import DataSink
from .utils import getdefaultconfig
from .... import logging

logger = logging.getLogger('workflow')


def create_skullstripped_recon_flow(name="skullstripped_recon_all"):
    """Performs recon-all on voulmes that are already skull stripped.
    FreeSurfer failes to perform skullstrippig on some volumes (especially
    MP2RAGE). This can be avoided by doing skullstripping before running
    recon-all (using for example SPECTRE algorithm).

    Example
    -------
    >>> from nipype.workflows.smri.freesurfer import create_skullstripped_recon_flow
    >>> recon_flow = create_skullstripped_recon_flow()
    >>> recon_flow.inputs.inputspec.subject_id = 'subj1'
    >>> recon_flow.inputs.inputspec.T1_files = 'T1.nii.gz'
    >>> recon_flow.run()  # doctest: +SKIP


    Inputs::
           inputspec.T1_files : skullstripped T1_files (mandatory)
           inputspec.subject_id : freesurfer subject id (optional)
           inputspec.subjects_dir : freesurfer subjects directory (optional)

    Outputs::

           outputspec.subject_id : freesurfer subject id
           outputspec.subjects_dir : freesurfer subjects directory
    """
    wf = pe.Workflow(name=name)

    inputnode = pe.Node(
        niu.IdentityInterface(
            fields=['subject_id', 'subjects_dir', 'T1_files']),
        name='inputspec')

    autorecon1 = pe.Node(fs.ReconAll(), name="autorecon1")
    autorecon1.plugin_args = {'submit_specs': 'request_memory = 2500'}
    autorecon1.inputs.directive = "autorecon1"
    autorecon1.inputs.args = "-noskullstrip"
    autorecon1._interface._can_resume = False

    wf.connect(inputnode, "T1_files", autorecon1, "T1_files")
    wf.connect(inputnode, "subjects_dir", autorecon1, "subjects_dir")
    wf.connect(inputnode, "subject_id", autorecon1, "subject_id")

    def link_masks(subjects_dir, subject_id):
        import os
        os.symlink(
            os.path.join(subjects_dir, subject_id, "mri", "T1.mgz"),
            os.path.join(subjects_dir, subject_id, "mri",
                         "brainmask.auto.mgz"))
        os.symlink(
            os.path.join(subjects_dir, subject_id, "mri",
                         "brainmask.auto.mgz"),
            os.path.join(subjects_dir, subject_id, "mri", "brainmask.mgz"))
        return subjects_dir, subject_id

    masks = pe.Node(
        niu.Function(
            input_names=['subjects_dir', 'subject_id'],
            output_names=['subjects_dir', 'subject_id'],
            function=link_masks),
        name="link_masks")

    wf.connect(autorecon1, "subjects_dir", masks, "subjects_dir")
    wf.connect(autorecon1, "subject_id", masks, "subject_id")

    autorecon_resume = pe.Node(fs.ReconAll(), name="autorecon_resume")
    autorecon_resume.plugin_args = {'submit_specs': 'request_memory = 2500'}
    autorecon_resume.inputs.args = "-no-isrunning"
    wf.connect(masks, "subjects_dir", autorecon_resume, "subjects_dir")
    wf.connect(masks, "subject_id", autorecon_resume, "subject_id")

    outputnode = pe.Node(
        niu.IdentityInterface(fields=['subject_id', 'subjects_dir']),
        name='outputspec')

    wf.connect(autorecon_resume, "subjects_dir", outputnode, "subjects_dir")
    wf.connect(autorecon_resume, "subject_id", outputnode, "subject_id")
    return wf


def create_reconall_workflow(name="ReconAll", plugin_args=None):
    """Creates the ReconAll workflow in Nipype. This workflow is designed to
    run the same commands as FreeSurfer's reconall script but with the added
    features that a Nipype workflow provides. Before running this workflow, it
    is necessary to have the FREESURFER_HOME environmental variable set to the
    directory containing the version of FreeSurfer to be used in this workflow.

    Example
    -------
    >>> from nipype.workflows.smri.freesurfer import create_reconall_workflow
    >>> recon_all = create_reconall_workflow()
    >>> recon_all.inputs.inputspec.subject_id = 'subj1'
    >>> recon_all.inputs.inputspec.subjects_dir = '.'
    >>> recon_all.inputs.inputspec.T1_files = 'T1.nii.gz'
    >>> recon_flow.run()  # doctest: +SKIP


    Inputs::
           inputspec.subjects_dir : subjects directory (mandatory)
           inputspec.subject_id : name of subject (mandatory)
           inputspec.T1_files : T1 files (mandatory)
           inputspec.T2_file : T2 file (optional)
           inputspec.FLAIR_file : FLAIR file (optional)
           inputspec.cw256 : Conform inputs to 256 FOV (optional)
           inputspec.num_threads: Number of threads on nodes that utilize OpenMP (default=1)
           plugin_args : Dictionary of plugin args to set to nodes that utilize OpenMP (optional)
    Outputs::
           postdatasink_outputspec.subject_id : name of the datasinked output folder in the subjects directory

    Note:
    The input subject_id is not passed to the commands in the workflow. Commands
    that require subject_id are reading implicit inputs from
    {SUBJECTS_DIR}/{subject_id}. For those commands the subject_id is set to the
    default value and SUBJECTS_DIR is set to the node directory. The implicit
    inputs are then copied to the node directory in order to mimic a SUBJECTS_DIR
    structure. For example, if the command implicitly reads in brainmask.mgz, the
    interface would copy that input file to
    {node_dir}/{subject_id}/mri/brainmask.mgz and set SUBJECTS_DIR to node_dir.
    The workflow only uses the input subject_id to datasink the outputs to
    {subjects_dir}/{subject_id}.
    """
    reconall = pe.Workflow(name=name)

    inputspec = pe.Node(
        niu.IdentityInterface(fields=[
            'subject_id', 'subjects_dir', 'T1_files', 'T2_file', 'FLAIR_file',
            'num_threads', 'cw256', 'reg_template', 'reg_template_withskull',
            'lh_atlas', 'rh_atlas', 'lh_classifier1', 'rh_classifier1',
            'lh_classifier2', 'rh_classifier2', 'lh_classifier3',
            'rh_classifier3', 'lookup_table', 'wm_lookup_table',
            'src_subject_id', 'src_subject_dir', 'color_table', 'awk_file'
        ]),
        run_without_submitting=True,
        name='inputspec')

    # check freesurfer version and set parameters
    fs_version_full = Info.version()
    if fs_version_full and ('v6.0' in fs_version_full
                            or 'dev' in fs_version_full):
        # assuming that dev is 6.0
        fsvernum = 6.0
        fs_version = 'v6.0'
        th3 = True
        shrink = 2
        distance = 200  # 3T should be 50
        stop = 0.0001
        exvivo = True
        entorhinal = True
        rb_date = "2014-08-21"
    else:
        # 5.3 is default
        fsvernum = 5.3
        if fs_version_full:
            if 'v5.3' in fs_version_full:
                fs_version = 'v5.3'
            else:
                fs_version = fs_version_full.split('-')[-1]
                logger.info(("Warning: Workflow may not work properly if "
                             "FREESURFER_HOME environmental variable is not "
                             "set or if you are using an older version of "
                             "FreeSurfer"))
        else:
            fs_version = 5.3  # assume version 5.3
        th3 = False
        shrink = None
        distance = 50
        stop = None
        exvivo = False
        entorhinal = False
        rb_date = "2008-03-26"

    logger.info("FreeSurfer Version: {0}".format(fs_version))

    def setconfig(reg_template=None,
                  reg_template_withskull=None,
                  lh_atlas=None,
                  rh_atlas=None,
                  lh_classifier1=None,
                  rh_classifier1=None,
                  lh_classifier2=None,
                  rh_classifier2=None,
                  lh_classifier3=None,
                  rh_classifier3=None,
                  src_subject_id=None,
                  src_subject_dir=None,
                  color_table=None,
                  lookup_table=None,
                  wm_lookup_table=None,
                  awk_file=None,
                  rb_date=None):
        """Set optional configurations to the default"""

        def checkarg(arg, default):
            """Returns the value if defined; otherwise default"""
            if arg:
                return arg
            else:
                return default

        defaultconfig = getdefaultconfig(exitonfail=True, rb_date=rb_date)
        # set the default template and classifier files
        reg_template = checkarg(reg_template,
                                defaultconfig['registration_template'])
        reg_template_withskull = checkarg(
            reg_template_withskull,
            defaultconfig['registration_template_withskull'])
        lh_atlas = checkarg(lh_atlas, defaultconfig['lh_atlas'])
        rh_atlas = checkarg(rh_atlas, defaultconfig['rh_atlas'])
        lh_classifier1 = checkarg(lh_classifier1,
                                  defaultconfig['lh_classifier'])
        rh_classifier1 = checkarg(rh_classifier1,
                                  defaultconfig['rh_classifier'])
        lh_classifier2 = checkarg(lh_classifier2,
                                  defaultconfig['lh_classifier2'])
        rh_classifier2 = checkarg(rh_classifier2,
                                  defaultconfig['rh_classifier2'])
        lh_classifier3 = checkarg(lh_classifier3,
                                  defaultconfig['lh_classifier3'])
        rh_classifier3 = checkarg(rh_classifier3,
                                  defaultconfig['rh_classifier3'])
        src_subject_id = checkarg(src_subject_id,
                                  defaultconfig['src_subject_id'])
        src_subject_dir = checkarg(src_subject_dir,
                                   defaultconfig['src_subject_dir'])
        color_table = checkarg(color_table, defaultconfig['AvgColorTable'])
        lookup_table = checkarg(lookup_table, defaultconfig['LookUpTable'])
        wm_lookup_table = checkarg(wm_lookup_table,
                                   defaultconfig['WMLookUpTable'])
        awk_file = checkarg(awk_file, defaultconfig['awk_file'])
        return reg_template, reg_template_withskull, lh_atlas, rh_atlas, \
            lh_classifier1, rh_classifier1, lh_classifier2, rh_classifier2, \
            lh_classifier3, rh_classifier3, src_subject_id, src_subject_dir, \
            color_table, lookup_table, wm_lookup_table, awk_file

    # list of params to check
    params = [
        'reg_template', 'reg_template_withskull', 'lh_atlas', 'rh_atlas',
        'lh_classifier1', 'rh_classifier1', 'lh_classifier2', 'rh_classifier2',
        'lh_classifier3', 'rh_classifier3', 'src_subject_id',
        'src_subject_dir', 'color_table', 'lookup_table', 'wm_lookup_table',
        'awk_file'
    ]

    config_node = pe.Node(
        niu.Function(params + ['rb_date'], params, setconfig), name="config")

    config_node.inputs.rb_date = rb_date

    for param in params:
        reconall.connect(inputspec, param, config_node, param)

    # create AutoRecon1
    ar1_wf, ar1_outputs = create_AutoRecon1(
        plugin_args=plugin_args,
        stop=stop,
        distance=distance,
        shrink=shrink,
        fsvernum=fsvernum)
    # connect inputs for AutoRecon1
    reconall.connect([(inputspec, ar1_wf, [
        ('T1_files', 'inputspec.T1_files'), ('T2_file', 'inputspec.T2_file'),
        ('FLAIR_file', 'inputspec.FLAIR_file'),
        ('num_threads', 'inputspec.num_threads'), ('cw256', 'inputspec.cw256')
    ]), (config_node, ar1_wf, [('reg_template_withskull',
                                'inputspec.reg_template_withskull'),
                               ('awk_file', 'inputspec.awk_file')])])
    # create AutoRecon2
    ar2_wf, ar2_outputs = create_AutoRecon2(
        plugin_args=plugin_args,
        fsvernum=fsvernum,
        stop=stop,
        shrink=shrink,
        distance=distance)
    # connect inputs for AutoRecon2
    reconall.connect(
        [(inputspec, ar2_wf, [('num_threads', 'inputspec.num_threads')]),
         (config_node, ar2_wf, [('reg_template_withskull',
                                 'inputspec.reg_template_withskull'),
                                ('reg_template', 'inputspec.reg_template')]),
         (ar1_wf, ar2_wf, [('outputspec.brainmask', 'inputspec.brainmask'),
                           ('outputspec.talairach', 'inputspec.transform'),
                           ('outputspec.orig', 'inputspec.orig')])])

    if fsvernum < 6:
        reconall.connect([(ar1_wf, ar2_wf, [('outputspec.nu',
                                             'inputspec.nu')])])

    # create AutoRecon3
    ar3_wf, ar3_outputs = create_AutoRecon3(
        plugin_args=plugin_args,
        th3=th3,
        exvivo=exvivo,
        entorhinal=entorhinal,
        fsvernum=fsvernum)
    # connect inputs for AutoRecon3
    reconall.connect(
        [(config_node, ar3_wf,
          [('lh_atlas', 'inputspec.lh_atlas'),
           ('rh_atlas', 'inputspec.rh_atlas'), ('lh_classifier1',
                                                'inputspec.lh_classifier1'),
           ('rh_classifier1',
            'inputspec.rh_classifier1'), ('lh_classifier2',
                                          'inputspec.lh_classifier2'),
           ('rh_classifier2',
            'inputspec.rh_classifier2'), ('lh_classifier3',
                                          'inputspec.lh_classifier3'),
           ('rh_classifier3',
            'inputspec.rh_classifier3'), ('lookup_table',
                                          'inputspec.lookup_table'),
           ('wm_lookup_table',
            'inputspec.wm_lookup_table'), ('src_subject_dir',
                                           'inputspec.src_subject_dir'),
           ('src_subject_id',
            'inputspec.src_subject_id'), ('color_table',
                                          'inputspec.color_table')]),
         (ar1_wf, ar3_wf, [('outputspec.brainmask', 'inputspec.brainmask'),
                           ('outputspec.talairach', 'inputspec.transform'),
                           ('outputspec.orig',
                            'inputspec.orig_mgz'), ('outputspec.rawavg',
                                                    'inputspec.rawavg')]),
         (ar2_wf, ar3_wf,
          [('outputspec.aseg_presurf', 'inputspec.aseg_presurf'),
           ('outputspec.brain_finalsurfs',
            'inputspec.brain_finalsurfs'), ('outputspec.wm', 'inputspec.wm'),
           ('outputspec.filled', 'inputspec.filled'), ('outputspec.norm',
                                                       'inputspec.norm')])])
    for hemi in ('lh', 'rh'):
        reconall.connect([(ar2_wf, ar3_wf,
                           [('outputspec.{0}_inflated'.format(hemi),
                             'inputspec.{0}_inflated'.format(hemi)),
                            ('outputspec.{0}_smoothwm'.format(hemi),
                             'inputspec.{0}_smoothwm'.format(hemi)),
                            ('outputspec.{0}_white'.format(hemi),
                             'inputspec.{0}_white'.format(hemi)),
                            ('outputspec.{0}_cortex'.format(hemi),
                             'inputspec.{0}_cortex_label'.format(hemi)),
                            ('outputspec.{0}_area'.format(hemi),
                             'inputspec.{0}_area'.format(hemi)),
                            ('outputspec.{0}_curv'.format(hemi),
                             'inputspec.{0}_curv'.format(hemi)),
                            ('outputspec.{0}_sulc'.format(hemi),
                             'inputspec.{0}_sulc'.format(hemi)),
                            ('outputspec.{0}_orig_nofix'.format(hemi),
                             'inputspec.{0}_orig_nofix'.format(hemi)),
                            ('outputspec.{0}_orig'.format(hemi),
                             'inputspec.{0}_orig'.format(hemi)),
                            ('outputspec.{0}_white_H'.format(hemi),
                             'inputspec.{0}_white_H'.format(hemi)),
                            ('outputspec.{0}_white_K'.format(hemi),
                             'inputspec.{0}_white_K'.format(hemi))])])

    # Add more outputs to outputspec
    outputs = ar1_outputs + ar2_outputs + ar3_outputs
    outputspec = pe.Node(
        niu.IdentityInterface(fields=outputs, mandatory_inputs=True),
        name="outputspec")

    for outfields, wf in [(ar1_outputs, ar1_wf), (ar2_outputs, ar2_wf),
                          (ar3_outputs, ar3_wf)]:
        for field in outfields:
            reconall.connect([(wf, outputspec, [('outputspec.' + field,
                                                 field)])])

    # PreDataSink: Switch Transforms to datasinked transfrom
    # The transforms in the header files of orig.mgz, orig_nu.mgz, and nu.mgz
    # are all reference a transform in the cache directory. We need to rewrite the
    # headers to reference the datasinked transform

    # get the filepath to where the transform will be datasinked
    def getDSTransformPath(subjects_dir, subject_id):
        import os
        transform = os.path.join(subjects_dir, subject_id, 'mri', 'transforms',
                                 'talairach.xfm')
        return transform

    dstransform = pe.Node(
        niu.Function(['subjects_dir', 'subject_id'], ['transform'],
                     getDSTransformPath),
        name="PreDataSink_GetTransformPath")
    reconall.connect([(inputspec, dstransform,
                       [('subjects_dir', 'subjects_dir'), ('subject_id',
                                                           'subject_id')])])
    # add the data sink transfrom location to the headers
    predatasink_orig = pe.Node(AddXFormToHeader(), name="PreDataSink_Orig")
    predatasink_orig.inputs.copy_name = True
    predatasink_orig.inputs.out_file = 'orig.mgz'
    reconall.connect([(outputspec, predatasink_orig, [('orig', 'in_file')]),
                      (dstransform, predatasink_orig, [('transform',
                                                        'transform')])])
    predatasink_orig_nu = pe.Node(
        AddXFormToHeader(), name="PreDataSink_Orig_Nu")
    predatasink_orig_nu.inputs.copy_name = True
    predatasink_orig_nu.inputs.out_file = 'orig_nu.mgz'
    reconall.connect(
        [(outputspec, predatasink_orig_nu, [('orig_nu', 'in_file')]),
         (dstransform, predatasink_orig_nu, [('transform', 'transform')])])
    predatasink_nu = pe.Node(AddXFormToHeader(), name="PreDataSink_Nu")
    predatasink_nu.inputs.copy_name = True
    predatasink_nu.inputs.out_file = 'nu.mgz'
    reconall.connect([(outputspec, predatasink_nu, [('nu', 'in_file')]),
                      (dstransform, predatasink_nu, [('transform',
                                                      'transform')])])

    # Datasink outputs
    datasink = pe.Node(DataSink(), name="DataSink")
    datasink.inputs.parameterization = False

    reconall.connect([(inputspec, datasink,
                       [('subjects_dir', 'base_directory'), ('subject_id',
                                                             'container')])])

    # assign datasink inputs
    reconall.connect([
        (predatasink_orig, datasink, [('out_file', 'mri.@orig')]),
        (predatasink_orig_nu, datasink, [('out_file', 'mri.@orig_nu')]),
        (predatasink_nu, datasink, [('out_file', 'mri.@nu')]),
        (outputspec, datasink, [
            ('origvols', 'mri.orig'),
            ('t2_raw', 'mri.orig.@t2raw'),
            ('flair', 'mri.orig.@flair'),
            ('rawavg', 'mri.@rawavg'),
            ('talairach_auto', 'mri.transforms.@tal_auto'),
            ('talairach', 'mri.transforms.@tal'),
            ('t1', 'mri.@t1'),
            ('brainmask_auto', 'mri.@brainmask_auto'),
            ('brainmask', 'mri.@brainmask'),
            ('braintemplate', 'mri.@braintemplate'),
            ('tal_lta', 'mri.transforms.@tal_lta'),
            ('norm', 'mri.@norm'),
            ('ctrl_pts', 'mri.@ctrl_pts'),
            ('tal_m3z', 'mri.transforms.@tal_m3z'),
            ('nu_noneck', 'mri.@nu_noneck'),
            ('talskull2', 'mri.transforms.@talskull2'),
            ('aseg_noCC', 'mri.@aseg_noCC'),
            ('cc_up', 'mri.transforms.@cc_up'),
            ('aseg_auto', 'mri.@aseg_auto'),
            ('aseg_presurf', 'mri.@aseg_presuf'),
            ('brain', 'mri.@brain'),
            ('brain_finalsurfs', 'mri.@brain_finalsurfs'),
            ('wm_seg', 'mri.@wm_seg'),
            ('wm_aseg', 'mri.@wm_aseg'),
            ('wm', 'mri.@wm'),
            ('filled', 'mri.@filled'),
            ('ponscc_log', 'mri.@ponscc_log'),
            ('lh_orig_nofix', 'surf.@lh_orig_nofix'),
            ('lh_orig', 'surf.@lh_orig'),
            ('lh_smoothwm_nofix', 'surf.@lh_smoothwm_nofix'),
            ('lh_inflated_nofix', 'surf.@lh_inflated_nofix'),
            ('lh_qsphere_nofix', 'surf.@lh_qsphere_nofix'),
            ('lh_white', 'surf.@lh_white'),
            ('lh_curv', 'surf.@lh_curv'),
            ('lh_area', 'surf.@lh_area'),
            ('lh_cortex', 'label.@lh_cortex'),
            ('lh_smoothwm', 'surf.@lh_smoothwm'),
            ('lh_sulc', 'surf.@lh_sulc'),
            ('lh_inflated', 'surf.@lh_inflated'),
            ('lh_white_H', 'surf.@lh_white_H'),
            ('lh_white_K', 'surf.@lh_white_K'),
            ('lh_inflated_H', 'surf.@lh_inflated_H'),
            ('lh_inflated_K', 'surf.@lh_inflated_K'),
            ('lh_curv_stats', 'stats.@lh_curv_stats'),
            ('rh_orig_nofix', 'surf.@rh_orig_nofix'),
            ('rh_orig', 'surf.@rh_orig'),
            ('rh_smoothwm_nofix', 'surf.@rh_smoothwm_nofix'),
            ('rh_inflated_nofix', 'surf.@rh_inflated_nofix'),
            ('rh_qsphere_nofix', 'surf.@rh_qsphere_nofix'),
            ('rh_white', 'surf.@rh_white'),
            ('rh_curv', 'surf.@rh_curv'),
            ('rh_area', 'surf.@rh_area'),
            ('rh_cortex', 'label.@rh_cortex'),
            ('rh_smoothwm', 'surf.@rh_smoothwm'),
            ('rh_sulc', 'surf.@rh_sulc'),
            ('rh_inflated', 'surf.@rh_inflated'),
            ('rh_white_H', 'surf.@rh_white_H'),
            ('rh_white_K', 'surf.@rh_white_K'),
            ('rh_inflated_H', 'surf.@rh_inflated_H'),
            ('rh_inflated_K', 'surf.@rh_inflated_K'),
            ('rh_curv_stats', 'stats.@rh_curv_stats'),
            ('lh_aparc_annot_ctab', 'label.@aparc_annot_ctab'),
            ('aseg', 'mri.@aseg'),
            ('wmparc', 'mri.@wmparc'),
            ('wmparc_stats', 'stats.@wmparc_stats'),
            ('aseg_stats', 'stats.@aseg_stats'),
            ('aparc_a2009s_aseg', 'mri.@aparc_a2009s_aseg'),
            ('aparc_aseg', 'mri.@aparc_aseg'),
            ('aseg_presurf_hypos', 'mri.@aseg_presurf_hypos'),
            ('ribbon', 'mri.@ribbon'),
            ('rh_ribbon', 'mri.@rh_ribbon'),
            ('lh_ribbon', 'mri.@lh_ribbon'),
            ('lh_sphere', 'surf.@lh_sphere'),
            ('rh_sphere', 'surf.@rh_sphere'),
            ('lh_sphere_reg', 'surf.@lh_sphere_reg'),
            ('rh_sphere_reg', 'surf.@rh_sphere_reg'),
            ('lh_jacobian_white', 'surf.@lh_jacobian_white'),
            ('rh_jacobian_white', 'surf.@rh_jacobian_white'),
            ('lh_avg_curv', 'surf.@lh_avg_curv'),
            ('rh_avg_curv', 'surf.@rh_avg_curv'),
            ('lh_aparc_annot', 'label.@lh_aparc_annot'),
            ('rh_aparc_annot', 'label.@rh_aparc_annot'),
            ('lh_area_pial', 'surf.@lh_area_pial'),
            ('rh_area_pial', 'surf.@rh_area_pial'),
            ('lh_curv_pial', 'surf.@lh_curv_pial'),
            ('rh_curv_pial', 'surf.@rh_curv_pial'),
            ('lh_pial', 'surf.@lh_pial'),
            ('rh_pial', 'surf.@rh_pial'),
            ('lh_thickness_pial', 'surf.@lh_thickness_pial'),
            ('rh_thickness_pial', 'surf.@rh_thickness_pial'),
            ('lh_area_mid', 'surf.@lh_area_mid'),
            ('rh_area_mid', 'surf.@rh_area_mid'),
            ('lh_volume', 'surf.@lh_volume'),
            ('rh_volume', 'surf.@rh_volume'),
            ('lh_aparc_annot_ctab', 'label.@lh_aparc_annot_ctab'),
            ('rh_aparc_annot_ctab', 'label.@rh_aparc_annot_ctab'),
            ('lh_aparc_stats', 'stats.@lh_aparc_stats'),
            ('rh_aparc_stats', 'stats.@rh_aparc_stats'),
            ('lh_aparc_pial_stats', 'stats.@lh_aparc_pial_stats'),
            ('rh_aparc_pial_stats', 'stats.@rh_aparc_pial_stats'),
            ('lh_aparc_a2009s_annot', 'label.@lh_aparc_a2009s_annot'),
            ('rh_aparc_a2009s_annot', 'label.@rh_aparc_a2009s_annot'),
            ('lh_aparc_a2009s_annot_ctab',
             'label.@lh_aparc_a2009s_annot_ctab'),
            ('rh_aparc_a2009s_annot_ctab',
             'label.@rh_aparc_a2009s_annot_ctab'),
            ('lh_aparc_a2009s_annot_stats',
             'stats.@lh_aparc_a2009s_annot_stats'),
            ('rh_aparc_a2009s_annot_stats',
             'stats.@rh_aparc_a2009s_annot_stats'),
            ('lh_aparc_DKTatlas40_annot', 'label.@lh_aparc_DKTatlas40_annot'),
            ('rh_aparc_DKTatlas40_annot', 'label.@rh_aparc_DKTatlas40_annot'),
            ('lh_aparc_DKTatlas40_annot_ctab',
             'label.@lh_aparc_DKTatlas40_annot_ctab'),
            ('rh_aparc_DKTatlas40_annot_ctab',
             'label.@rh_aparc_DKTatlas40_annot_ctab'),
            ('lh_aparc_DKTatlas40_annot_stats',
             'stats.@lh_aparc_DKTatlas40_annot_stats'),
            ('rh_aparc_DKTatlas40_annot_stats',
             'stats.@rh_aparc_DKTatlas40_annot_stats'),
            ('lh_wg_pct_mgh', 'surf.@lh_wg_pct_mgh'),
            ('rh_wg_pct_mgh', 'surf.@rh_wg_pct_mgh'),
            ('lh_wg_pct_stats', 'stats.@lh_wg_pct_stats'),
            ('rh_wg_pct_stats', 'stats.@rh_wg_pct_stats'),
            ('lh_pctsurfcon_log', 'log.@lh_pctsurfcon_log'),
            ('rh_pctsurfcon_log', 'log.@rh_pctsurfcon_log'),
            ('lh_BAMaps_stats', 'stats.@lh_BAMaps_stats'),
            ('lh_color', 'label.@lh_color'),
            ('lh_thresh_BAMaps_stats', 'stats.@lh_thresh_BAMaps_stats'),
            ('lh_thresh_color', 'label.@lh_thresh_color'),
            ('rh_BAMaps_stats', 'stats.@rh_BAMaps_stats'),
            ('rh_color', 'label.@rh_color'),
            ('rh_thresh_BAMaps_stats', 'stats.@rh_thresh_BAMaps_stats'),
            ('rh_thresh_color', 'label.@rh_thresh_color'),
            ('lh_BAMaps_labels', 'label.@lh_BAMaps_labels'),
            ('lh_thresh_BAMaps_labels', 'label.@lh_thresh_BAMaps_labels'),
            ('rh_BAMaps_labels', 'label.@rh_BAMaps_labels'),
            ('rh_thresh_BAMaps_labels', 'label.@rh_thresh_BAMaps_labels'),
            ('lh_BAMaps_annotation', 'label.@lh_BAMaps_annotation'),
            ('lh_thresh_BAMaps_annotation',
             'label.@lh_thresh_BAMaps_annotation'),
            ('rh_BAMaps_annotation', 'label.@rh_BAMaps_annotation'),
            ('rh_thresh_BAMaps_annotation',
             'label.@rh_thresh_BAMaps_annotation'),
        ]),
    ])

    # compeltion node
    # since recon-all outputs so many files a completion node is added
    # that will output the subject_id once the workflow has completed
    def completemethod(datasinked_files, subject_id):
        print("recon-all has finished executing for subject: {0}".format(
            subject_id))
        return subject_id

    completion = pe.Node(
        niu.Function(['datasinked_files', 'subject_id'], ['subject_id'],
                     completemethod),
        name="Completion")

    # create a special identity interface for outputing the subject_id

    postds_outputspec = pe.Node(
        niu.IdentityInterface(['subject_id']), name="postdatasink_outputspec")

    reconall.connect(
        [(datasink, completion, [('out_file', 'datasinked_files')]),
         (inputspec, completion, [('subject_id', 'subject_id')]),
         (completion, postds_outputspec, [('subject_id', 'subject_id')])])

    return reconall



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..tensors import GenerateDirections


def test_GenerateDirections_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        display_debug=dict(argstr='-debug', ),
        display_info=dict(argstr='-info', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        ignore_exception=dict(
            deprecated='1.0.0',
            nohash=True,
            usedefault=True,
        ),
        niter=dict(argstr='-niter %s', ),
        num_dirs=dict(
            argstr='%s',
            mandatory=True,
            position=-2,
        ),
        out_file=dict(
            argstr='%s',
            hash_files=False,
            name_source=['num_dirs'],
            name_template='directions_%d.txt',
            position=-1,
        ),
        power=dict(argstr='-power %s', ),
        quiet_display=dict(argstr='-quiet', ),
        terminal_output=dict(
            deprecated='1.0.0',
            nohash=True,
        ),
    )
    inputs = GenerateDirections.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_GenerateDirections_outputs():
    output_map = dict(out_file=dict(), )
    outputs = GenerateDirections.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



import os
import pytest
import shutil

from nipype.interfaces.dcm2nii import Dcm2niix
no_dcm2niix = not bool(Dcm2niix().version)
no_datalad = False
try:
    from datalad import api # to pull and grab data
    from datalad.support.exceptions import IncompleteResultsError
except ImportError:
    no_datalad = True

DICOM_DIR = 'http://datasets-tests.datalad.org/dicoms/dcm2niix-tests'


def fetch_data(tmpdir, dicoms):
    """Fetches some test DICOMs using datalad"""
    data = os.path.join(tmpdir, 'data')
    api.install(path=data, source=DICOM_DIR)
    data = os.path.join(data, dicoms)
    api.get(path=data)
    return data

@pytest.mark.skipif(no_datalad, reason="Datalad required")
@pytest.mark.skipif(no_dcm2niix, reason="Dcm2niix required")
def test_dcm2niix_dwi(tmpdir):
    tmpdir.chdir()
    try:
        datadir = fetch_data(tmpdir.strpath, 'Siemens_Sag_DTI_20160825_145811')
    except IncompleteResultsError as exc:
        pytest.skip("Failed to fetch test data: %s" % str(exc))

    def assert_dwi(eg, bids):
        "Some assertions we will make"
        assert eg.outputs.converted_files
        assert eg.outputs.bvals
        assert eg.outputs.bvecs
        outputs = [y for x,y in eg.outputs.get().items()]
        if bids:
            # ensure all outputs are of equal lengths
            assert len(set(map(len, outputs))) == 1
        else:
            assert not eg2.outputs.bids

    dcm = Dcm2niix()
    dcm.inputs.source_dir = datadir
    dcm.inputs.out_filename = '%u%z'
    eg1 = dcm.run()
    assert_dwi(eg1, True)

    # now run specifying output directory and removing BIDS option
    outdir = tmpdir.mkdir('conversion').strpath
    dcm.inputs.output_dir = outdir
    dcm.inputs.bids_format = False
    eg2 = dcm.run()
    assert_dwi(eg2, False)



# -*- coding: utf-8 -*-
from .utils import (create_getmask_flow, create_get_stats_flow,
                    create_tessellation_flow)
from .bem import create_bem_flow
from .recon import create_skullstripped_recon_flow, create_reconall_workflow



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from __future__ import division

from copy import deepcopy
import os

from nibabel import Nifti1Image
import numpy as np

import pytest
import numpy.testing as npt
from nipype.interfaces.base import Bunch, TraitError
from nipype.algorithms.modelgen import (SpecifyModel, SpecifySparseModel,
                                        SpecifySPMModel)


def test_modelgen1(tmpdir):
    filename1 = tmpdir.join('test1.nii').strpath
    filename2 = tmpdir.join('test2.nii').strpath
    Nifti1Image(np.random.rand(10, 10, 10, 200),
                np.eye(4)).to_filename(filename1)
    Nifti1Image(np.random.rand(10, 10, 10, 200),
                np.eye(4)).to_filename(filename2)
    s = SpecifyModel()
    s.inputs.input_units = 'scans'
    set_output_units = lambda: setattr(s.inputs, 'output_units', 'scans')
    with pytest.raises(TraitError):
        set_output_units()
    s.inputs.functional_runs = [filename1, filename2]
    s.inputs.time_repetition = 6
    s.inputs.high_pass_filter_cutoff = 128.
    info = [
        Bunch(
            conditions=['cond1'],
            onsets=[[2, 50, 100, 180]],
            durations=[[1]],
            amplitudes=None,
            pmod=None,
            regressors=None,
            regressor_names=None,
            tmod=None),
        Bunch(
            conditions=['cond1'],
            onsets=[[30, 40, 100, 150]],
            durations=[[1]],
            amplitudes=None,
            pmod=None,
            regressors=None,
            regressor_names=None,
            tmod=None)
    ]
    s.inputs.subject_info = info
    res = s.run()
    assert len(res.outputs.session_info) == 2
    assert len(res.outputs.session_info[0]['regress']) == 0
    assert len(res.outputs.session_info[0]['cond']) == 1
    npt.assert_almost_equal(
        np.array(res.outputs.session_info[0]['cond'][0]['onset']),
        np.array([12, 300, 600, 1080]))
    info = [
        Bunch(conditions=['cond1'], onsets=[[2]], durations=[[1]]),
        Bunch(conditions=['cond1'], onsets=[[3]], durations=[[1]])
    ]
    s.inputs.subject_info = deepcopy(info)
    res = s.run()
    npt.assert_almost_equal(
        np.array(res.outputs.session_info[0]['cond'][0]['duration']),
        np.array([6.]))
    npt.assert_almost_equal(
        np.array(res.outputs.session_info[1]['cond'][0]['duration']),
        np.array([6.]))
    info = [
        Bunch(
            conditions=['cond1', 'cond2'],
            onsets=[[2, 3], [2]],
            durations=[[1, 1], [1]]),
        Bunch(
            conditions=['cond1', 'cond2'],
            onsets=[[2, 3], [2, 4]],
            durations=[[1, 1], [1, 1]])
    ]
    s.inputs.subject_info = deepcopy(info)
    s.inputs.input_units = 'scans'
    res = s.run()
    npt.assert_almost_equal(
        np.array(res.outputs.session_info[0]['cond'][0]['duration']),
        np.array([6., 6.]))
    npt.assert_almost_equal(
        np.array(res.outputs.session_info[0]['cond'][1]['duration']),
        np.array([
            6.,
        ]))
    npt.assert_almost_equal(
        np.array(res.outputs.session_info[1]['cond'][1]['duration']),
        np.array([6., 6.]))


def test_modelgen_spm_concat(tmpdir):
    filename1 = tmpdir.join('test1.nii').strpath
    filename2 = tmpdir.join('test2.nii').strpath
    Nifti1Image(np.random.rand(10, 10, 10, 30),
                np.eye(4)).to_filename(filename1)
    Nifti1Image(np.random.rand(10, 10, 10, 30),
                np.eye(4)).to_filename(filename2)

    # Test case when only one duration is passed, as being the same for all onsets.
    s = SpecifySPMModel()
    s.inputs.input_units = 'secs'
    s.inputs.concatenate_runs = True
    setattr(s.inputs, 'output_units', 'secs')
    assert s.inputs.output_units == 'secs'
    s.inputs.functional_runs = [filename1, filename2]
    s.inputs.time_repetition = 6
    s.inputs.high_pass_filter_cutoff = 128.
    info = [
        Bunch(
            conditions=['cond1'], onsets=[[2, 50, 100, 170]], durations=[[1]]),
        Bunch(
            conditions=['cond1'], onsets=[[30, 40, 100, 150]], durations=[[1]])
    ]
    s.inputs.subject_info = deepcopy(info)
    res = s.run()
    assert len(res.outputs.session_info) == 1
    assert len(res.outputs.session_info[0]['regress']) == 1
    assert np.sum(res.outputs.session_info[0]['regress'][0]['val']) == 30
    assert len(res.outputs.session_info[0]['cond']) == 1
    npt.assert_almost_equal(
        np.array(res.outputs.session_info[0]['cond'][0]['onset']),
        np.array([2.0, 50.0, 100.0, 170.0, 210.0, 220.0, 280.0, 330.0]))
    npt.assert_almost_equal(
        np.array(res.outputs.session_info[0]['cond'][0]['duration']),
        np.array([1., 1., 1., 1., 1., 1., 1., 1.]))

    # Test case of scans as output units instead of seconds
    setattr(s.inputs, 'output_units', 'scans')
    assert s.inputs.output_units == 'scans'
    s.inputs.subject_info = deepcopy(info)
    res = s.run()
    npt.assert_almost_equal(
        np.array(res.outputs.session_info[0]['cond'][0]['onset']),
        np.array([2.0, 50.0, 100.0, 170.0, 210.0, 220.0, 280.0, 330.0]) / 6)

    # Test case for no concatenation with seconds as output units
    s.inputs.concatenate_runs = False
    s.inputs.subject_info = deepcopy(info)
    s.inputs.output_units = 'secs'
    res = s.run()
    npt.assert_almost_equal(
        np.array(res.outputs.session_info[0]['cond'][0]['onset']),
        np.array([2.0, 50.0, 100.0, 170.0]))

    # Test case for variable number of events in separate runs, sometimes unique.
    filename3 = tmpdir.join('test3.nii').strpath
    Nifti1Image(np.random.rand(10, 10, 10, 30),
                np.eye(4)).to_filename(filename3)
    s.inputs.functional_runs = [filename1, filename2, filename3]
    info = [
        Bunch(
            conditions=['cond1', 'cond2'],
            onsets=[[2, 3], [2]],
            durations=[[1, 1], [1]]),
        Bunch(
            conditions=['cond1', 'cond2'],
            onsets=[[2, 3], [2, 4]],
            durations=[[1, 1], [1, 1]]),
        Bunch(
            conditions=['cond1', 'cond2'],
            onsets=[[2, 3], [2]],
            durations=[[1, 1], [1]])
    ]
    s.inputs.subject_info = deepcopy(info)
    res = s.run()
    npt.assert_almost_equal(
        np.array(res.outputs.session_info[0]['cond'][0]['duration']),
        np.array([1., 1.]))
    npt.assert_almost_equal(
        np.array(res.outputs.session_info[0]['cond'][1]['duration']),
        np.array([
            1.,
        ]))
    npt.assert_almost_equal(
        np.array(res.outputs.session_info[1]['cond'][1]['duration']),
        np.array([1., 1.]))
    npt.assert_almost_equal(
        np.array(res.outputs.session_info[2]['cond'][1]['duration']),
        np.array([
            1.,
        ]))

    # Test case for variable number of events in concatenated runs, sometimes unique.
    s.inputs.concatenate_runs = True
    info = [
        Bunch(
            conditions=['cond1', 'cond2'],
            onsets=[[2, 3], [2]],
            durations=[[1, 1], [1]]),
        Bunch(
            conditions=['cond1', 'cond2'],
            onsets=[[2, 3], [2, 4]],
            durations=[[1, 1], [1, 1]]),
        Bunch(
            conditions=['cond1', 'cond2'],
            onsets=[[2, 3], [2]],
            durations=[[1, 1], [1]])
    ]
    s.inputs.subject_info = deepcopy(info)
    res = s.run()
    npt.assert_almost_equal(
        np.array(res.outputs.session_info[0]['cond'][0]['duration']),
        np.array([1., 1., 1., 1., 1., 1.]))
    npt.assert_almost_equal(
        np.array(res.outputs.session_info[0]['cond'][1]['duration']),
        np.array([1., 1., 1., 1.]))


def test_modelgen_sparse(tmpdir):
    filename1 = tmpdir.join('test1.nii').strpath
    filename2 = tmpdir.join('test2.nii').strpath
    Nifti1Image(np.random.rand(10, 10, 10, 50),
                np.eye(4)).to_filename(filename1)
    Nifti1Image(np.random.rand(10, 10, 10, 50),
                np.eye(4)).to_filename(filename2)
    s = SpecifySparseModel()
    s.inputs.input_units = 'secs'
    s.inputs.functional_runs = [filename1, filename2]
    s.inputs.time_repetition = 6
    info = [
        Bunch(
            conditions=['cond1'], onsets=[[0, 50, 100, 180]], durations=[[2]]),
        Bunch(
            conditions=['cond1'], onsets=[[30, 40, 100, 150]], durations=[[1]])
    ]
    s.inputs.subject_info = info
    s.inputs.volumes_in_cluster = 1
    s.inputs.time_acquisition = 2
    s.inputs.high_pass_filter_cutoff = np.inf
    res = s.run()
    assert len(res.outputs.session_info) == 2
    assert len(res.outputs.session_info[0]['regress']) == 1
    assert len(res.outputs.session_info[0]['cond']) == 0

    s.inputs.stimuli_as_impulses = False
    res = s.run()
    assert res.outputs.session_info[0]['regress'][0]['val'][0] == 1.0

    s.inputs.model_hrf = True
    res = s.run()
    npt.assert_almost_equal(
        res.outputs.session_info[0]['regress'][0]['val'][0],
        0.016675298129743384)
    assert len(res.outputs.session_info[0]['regress']) == 1
    s.inputs.use_temporal_deriv = True
    res = s.run()

    assert len(res.outputs.session_info[0]['regress']) == 2
    npt.assert_almost_equal(
        res.outputs.session_info[0]['regress'][0]['val'][0],
        0.016675298129743384)
    npt.assert_almost_equal(
        res.outputs.session_info[1]['regress'][1]['val'][5],
        0.007671459162258378)



# -*- coding: utf-8 -*-
import os

import pytest
import nipype.workflows.fmri.fsl as fsl_wf
import nipype.interfaces.fsl as fsl
import nipype.interfaces.utility as util
from nipype.interfaces.fsl import no_fsl, no_fsl_course_data

import nipype.pipeline.engine as pe
import warnings
from nipype.workflows.dmri.fsl.epi import create_eddy_correct_pipeline


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
@pytest.mark.skipif(no_fsl_course_data(), reason="fsl data not available")
def test_create_eddy_correct_pipeline(tmpdir):
    fsl_course_dir = os.path.abspath(os.environ['FSL_COURSE_DATA'])

    dwi_file = os.path.join(fsl_course_dir, "fdt1/subj1/data.nii.gz")

    trim_dwi = pe.Node(fsl.ExtractROI(t_min=0, t_size=2), name="trim_dwi")
    trim_dwi.inputs.in_file = dwi_file

    nipype_eddycorrect = create_eddy_correct_pipeline("nipype_eddycorrect")
    nipype_eddycorrect.inputs.inputnode.ref_num = 0

    with warnings.catch_warnings():
        warnings.simplefilter("ignore")
        original_eddycorrect = pe.Node(
            interface=fsl.EddyCorrect(), name="original_eddycorrect")
    original_eddycorrect.inputs.ref_num = 0

    test = pe.Node(util.AssertEqual(), name="eddy_corrected_dwi_test")

    pipeline = pe.Workflow(name="test_eddycorrect")
    pipeline.base_dir = tmpdir.mkdir("nipype_test_eddycorrect_").strpath

    pipeline.connect([
        (trim_dwi, original_eddycorrect, [("roi_file", "in_file")]),
        (trim_dwi, nipype_eddycorrect, [("roi_file", "inputnode.in_file")]),
        (nipype_eddycorrect, test, [("outputnode.eddy_corrected", "volume1")]),
        (original_eddycorrect, test, [("eddy_corrected", "volume2")]),
    ])

    pipeline.run(plugin='Linear')



# -*- coding: utf-8 -*-
# coding: utf-8
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)

import os.path as op


def get_flirt_schedule(name):
    if name == 'ecc':
        return op.abspath(op.join(op.dirname(__file__), 'ecc.sch'))
    elif name == 'hmc':
        return op.abspath(op.join(op.dirname(__file__), 'hmc.sch'))
    else:
        raise RuntimeError('Requested file does not exist.')
